Class: Moxml::C14n::XmlBaseHandler
- Inherits:
-
Object
- Object
- Moxml::C14n::XmlBaseHandler
- Defined in:
- lib/moxml/c14n/xml_base_handler.rb
Overview
xml:base fixup handler for document subsets (C14N 1.1 2.4). Implements RFC 3986 URI joining with C14N 1.1 modifications.
Instance Method Summary collapse
-
#fixup_xml_base(element, omitted_ancestors) ⇒ Object
Returns the fixed-up xml:base value to emit on the element, or nil if no fixup is needed.
Instance Method Details
#fixup_xml_base(element, omitted_ancestors) ⇒ Object
Returns the fixed-up xml:base value to emit on the element, or nil if no fixup is needed.
12 13 14 15 16 17 18 19 |
# File 'lib/moxml/c14n/xml_base_handler.rb', line 12 def fixup_xml_base(element, omitted_ancestors) return nil if omitted_ancestors.empty? base_values = collect_base_values(element, omitted_ancestors) return nil if base_values.empty? join_base_values(base_values) end |