Class: Moxml::Adapter::Ox

Inherits:
Base
  • Object
show all
Defined in:
lib/moxml/adapter/ox.rb

Direct Known Subclasses

HeadedOx

Constant Summary

Constants inherited from Base

Base::ENTITY_MARKER, Base::ENTITY_MARKER_RE, Base::ENTITY_NAME_PATTERN, Base::ENTITY_NAME_RE, Base::SERIALIZED_ENTITY_MARKER_RE, Base::STANDARD_ENTITIES

Class Method Summary collapse

Methods inherited from Base

actual_native, create_cdata, create_comment, create_declaration, create_doctype, create_element, create_entity_reference, create_namespace, create_processing_instruction, create_text, in_scope_namespaces, prepare_for_new_document, preprocess_entities, restore_entities, sax_supported?

Methods included from XmlUtils

#encode_entities, #normalize_xml_value, #validate_comment_content, #validate_declaration_encoding, #validate_declaration_standalone, #validate_declaration_version, #validate_element_name, #validate_entity_reference_name, #validate_pi_target, #validate_prefix, #validate_uri

Class Method Details

.add_child(element, child) ⇒ Object



368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
# File 'lib/moxml/adapter/ox.rb', line 368

def add_child(element, child)
  # Special handling for declarations on Ox documents
  if element.is_a?(::Ox::Document) && child.is_a?(::Ox::Instruct) && child.target == "xml"
    # Transfer declaration attributes to document
    element.attributes ||= {}
    if child.attributes["version"]
      element.attributes[:version] =
        child.attributes["version"]
    end
    if child.attributes["encoding"]
      element.attributes[:encoding] =
        child.attributes["encoding"]
    end
    if child.attributes["standalone"]
      element.attributes[:standalone] =
        child.attributes["standalone"]
    end
  end

  child.parent = element if child.is_a?(::Ox::Node)
  element.nodes ||= []
  element.nodes << child

  # Mark document if EntityReference is added (avoids tree scan in serialize)
  if child.is_a?(::Moxml::Adapter::CustomizedOx::EntityReference)
    root = element
    while root.is_a?(::Ox::Node) && root.parent
      root = root.parent
    end
    attachments.set(root, :has_entity_refs, true) if root
  end
end

.add_next_sibling(node, sibling) ⇒ Object



412
413
414
415
416
417
418
419
420
421
# File 'lib/moxml/adapter/ox.rb', line 412

def add_next_sibling(node, sibling)
  return unless (parent = parent(node))

  if sibling.is_a?(::Ox::Node)
    sibling.parent&.nodes&.delete(sibling)
    sibling.parent = parent
  end
  idx = parent.nodes.index(node)
  parent.nodes.insert(idx + 1, sibling) if idx
end

.add_previous_sibling(node, sibling) ⇒ Object



401
402
403
404
405
406
407
408
409
410
# File 'lib/moxml/adapter/ox.rb', line 401

def add_previous_sibling(node, sibling)
  return unless (parent = parent(node))

  if sibling.is_a?(::Ox::Node)
    sibling.parent&.nodes&.delete(sibling)
    sibling.parent = parent
  end
  idx = parent.nodes.index(node)
  parent.nodes.insert(idx, sibling) if idx
end

.ancestors(node) ⇒ Object



172
173
174
175
176
# File 'lib/moxml/adapter/ox.rb', line 172

def ancestors(node)
  return [] unless (parent = parent(node))

  [parent] + ancestors(parent)
end

.assign_parents(node, parent = nil) ⇒ Object



464
465
466
467
468
469
470
471
# File 'lib/moxml/adapter/ox.rb', line 464

def assign_parents(node, parent = nil)
  node.parent = parent if node.respond_to?(:parent=) && parent
  return unless node.respond_to?(:nodes)

  node.nodes&.each do |child|
    assign_parents(child, node)
  end
end

.at_xpath(node, expression, namespaces = {}) ⇒ Object



620
621
622
# File 'lib/moxml/adapter/ox.rb', line 620

def at_xpath(node, expression, namespaces = {})
  xpath(node, expression, namespaces)&.first
end

.attachmentsObject



14
15
16
# File 'lib/moxml/adapter/ox.rb', line 14

def attachments
  @attachments ||= Moxml::NativeAttachment.new
end

.attribute_element(attribute) ⇒ Object



306
307
308
# File 'lib/moxml/adapter/ox.rb', line 306

def attribute_element(attribute)
  attribute.parent
end

.attributes(element) ⇒ Object



293
294
295
296
297
298
299
300
301
302
303
304
# File 'lib/moxml/adapter/ox.rb', line 293

def attributes(element)
  return [] unless element.is_a?(::Ox::Element) && element.attributes

  element.attributes.filter_map do |name, value|
    next if name.to_s.start_with?("xmlns")

    # Ensure value is passed correctly - Ox stores with symbol keys
    ::Moxml::Adapter::CustomizedOx::Attribute.new(
      name.to_s, value, element
    )
  end
end

.cdata_content(node) ⇒ Object



514
515
516
# File 'lib/moxml/adapter/ox.rb', line 514

def cdata_content(node)
  node.value.to_s
end

.children(node) ⇒ Object



251
252
253
254
255
256
257
258
259
260
261
# File 'lib/moxml/adapter/ox.rb', line 251

def children(node)
  return [] unless node.is_a?(::Ox::Element) || node.is_a?(::Ox::Document)

  result = node.nodes || []
  # Ox doesn't set parent references during parsing.
  # Set them here so parent/sibling navigation works.
  result.each do |child|
    child.parent = node if child.respond_to?(:parent=)
  end
  result
end

.comment_content(node) ⇒ Object



522
523
524
# File 'lib/moxml/adapter/ox.rb', line 522

def comment_content(node)
  node.value.to_s
end

.create_document(native_doc = nil) ⇒ Object



71
72
73
74
# File 'lib/moxml/adapter/ox.rb', line 71

def create_document(native_doc = nil)
  attrs = native_doc&.attributes || {}
  ::Ox::Document.new(**attrs)
end

.create_native_cdata(content, _owner_doc = nil) ⇒ Object



93
94
95
# File 'lib/moxml/adapter/ox.rb', line 93

def create_native_cdata(content, _owner_doc = nil)
  ::Ox::CData.new(content)
end

.create_native_comment(content, _owner_doc = nil) ⇒ Object



97
98
99
# File 'lib/moxml/adapter/ox.rb', line 97

def create_native_comment(content, _owner_doc = nil)
  ::Ox::Comment.new(content)
end

.create_native_declaration(version, encoding, standalone) ⇒ Object



113
114
115
116
117
118
119
# File 'lib/moxml/adapter/ox.rb', line 113

def create_native_declaration(version, encoding, standalone)
  inst = ::Ox::Instruct.new("xml")
  set_attribute(inst, "version", version)
  set_attribute(inst, "encoding", encoding)
  set_attribute(inst, "standalone", standalone)
  inst
end

.create_native_doctype(name, external_id, system_id) ⇒ Object



101
102
103
104
105
# File 'lib/moxml/adapter/ox.rb', line 101

def create_native_doctype(name, external_id, system_id)
  ::Ox::DocType.new(
    "#{name} PUBLIC \"#{external_id}\" \"#{system_id}\"",
  )
end

.create_native_element(name, _owner_doc = nil) ⇒ Object



76
77
78
79
# File 'lib/moxml/adapter/ox.rb', line 76

def create_native_element(name, _owner_doc = nil)
  element = ::Ox::Element.new(name)
  element
end

.create_native_entity_reference(name) ⇒ Object



85
86
87
# File 'lib/moxml/adapter/ox.rb', line 85

def create_native_entity_reference(name)
  ::Moxml::Adapter::CustomizedOx::EntityReference.new(name)
end

.create_native_namespace(element, prefix, uri) ⇒ Object



129
130
131
132
133
134
# File 'lib/moxml/adapter/ox.rb', line 129

def create_native_namespace(element, prefix, uri)
  ns = ::Moxml::Adapter::CustomizedOx::Namespace.new(prefix, uri,
                                                     element)
  set_attribute(element, ns.expanded_prefix, uri)
  ns
end

.create_native_processing_instruction(target, content) ⇒ Object



107
108
109
110
111
# File 'lib/moxml/adapter/ox.rb', line 107

def create_native_processing_instruction(target, content)
  inst = ::Ox::Instruct.new(target)
  set_processing_instruction_content(inst, content)
  inst
end

.create_native_text(content, _owner_doc = nil) ⇒ Object



81
82
83
# File 'lib/moxml/adapter/ox.rb', line 81

def create_native_text(content, _owner_doc = nil)
  content
end

.declaration_attribute(declaration, attr_name) ⇒ Object



121
122
123
# File 'lib/moxml/adapter/ox.rb', line 121

def declaration_attribute(declaration, attr_name)
  get_attribute_value(declaration, attr_name)
end

.doctype_external_id(native) ⇒ Object



570
571
572
573
574
575
# File 'lib/moxml/adapter/ox.rb', line 570

def doctype_external_id(native)
  value = native.value.to_s
  # Match PUBLIC "external_id"
  match = value.match(/PUBLIC\s+"([^"]*)"/)
  match ? match[1] : nil
end

.doctype_name(native) ⇒ Object

Doctype accessor methods Ox stores DOCTYPE as a string, so we parse it



563
564
565
566
567
568
# File 'lib/moxml/adapter/ox.rb', line 563

def doctype_name(native)
  # Parse: "name PUBLIC \"external_id\" \"system_id\"" or "name SYSTEM \"system_id\""
  value = native.value.to_s.strip
  # Extract the first word (the name)
  value.split(/\s+/).first
end

.doctype_system_id(native) ⇒ Object



577
578
579
580
581
582
583
584
# File 'lib/moxml/adapter/ox.rb', line 577

def doctype_system_id(native)
  value = native.value.to_s
  # Match the last quoted string (system_id)
  # For PUBLIC: "name PUBLIC \"external_id\" \"system_id\""
  # For SYSTEM: "name SYSTEM \"system_id\""
  matches = value.scan(/"([^"]*)"/)
  matches.last&.first
end

.document(node) ⇒ Object



283
284
285
286
287
# File 'lib/moxml/adapter/ox.rb', line 283

def document(node)
  current = node
  current = parent(current) while parent(current)
  current
end

.duplicate_node(node) ⇒ Object



216
217
218
# File 'lib/moxml/adapter/ox.rb', line 216

def duplicate_node(node)
  Marshal.load(Marshal.dump(node))
end

.entity_reference_name(node) ⇒ Object



89
90
91
# File 'lib/moxml/adapter/ox.rb', line 89

def entity_reference_name(node)
  node.name if node.is_a?(::Moxml::Adapter::CustomizedOx::EntityReference)
end

.get_attribute(element, name) ⇒ Object



343
344
345
346
347
348
349
350
351
352
353
354
355
# File 'lib/moxml/adapter/ox.rb', line 343

def get_attribute(element, name)
  return unless element.is_a?(::Ox::HasAttrs) && element.attributes
  unless element.attributes.key?(name.to_s) || element.attributes.key?(name.to_s.to_sym)
    return
  end

  # Ox stores attributes with symbol keys, so try both string and symbol
  value = element.attributes[name.to_s] || element.attributes[name.to_s.to_sym]

  ::Moxml::Adapter::CustomizedOx::Attribute.new(
    name.to_s, value, element
  )
end

.get_attribute_value(element, name) ⇒ Object



357
358
359
# File 'lib/moxml/adapter/ox.rb', line 357

def get_attribute_value(element, name)
  element[name]
end

.has_declaration?(native_doc, _wrapper) ⇒ Boolean

Returns:

  • (Boolean)


665
666
667
668
# File 'lib/moxml/adapter/ox.rb', line 665

def has_declaration?(native_doc, _wrapper)
  # Ox stores declaration in document attributes
  native_doc[:version] || native_doc[:encoding] || native_doc[:standalone]
end

.inner_text(node) ⇒ Object



499
500
501
502
503
# File 'lib/moxml/adapter/ox.rb', line 499

def inner_text(node)
  return "" unless node.is_a?(::Ox::Element) || node.is_a?(::Ox::Document)

  node.nodes.grep(String).join
end

.namespace(element) ⇒ Object



150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
# File 'lib/moxml/adapter/ox.rb', line 150

def namespace(element)
  prefix =
    if element.is_a?(::Moxml::Adapter::CustomizedOx::Attribute)
      element.prefix
    elsif element.name.include?(":")
      element.name.split(":").first
    end
  attr_name = ["xmlns", prefix].compact.join(":")

  ([element] + ancestors(element)).each do |node|
    next unless node.is_a?(::Ox::Element) && node.attributes

    if node[attr_name]
      return ::Moxml::Adapter::CustomizedOx::Namespace.new(
        prefix, node[attr_name], element
      )
    end
  end

  nil
end

.namespace_definitions(node) ⇒ Object



546
547
548
549
550
551
552
553
554
555
556
557
558
559
# File 'lib/moxml/adapter/ox.rb', line 546

def namespace_definitions(node)
  return [] unless node.is_a?(::Ox::Element) && node.attributes

  namespaces = {}
  node.attributes.each do |name, value|
    name_s = name.to_s
    next unless name_s == "xmlns" || name_s.start_with?("xmlns:")

    namespaces[name] = ::Moxml::Adapter::CustomizedOx::Namespace.new(
      name, value, node
    )
  end
  namespaces.values
end

.namespace_prefix(namespace) ⇒ Object



538
539
540
# File 'lib/moxml/adapter/ox.rb', line 538

def namespace_prefix(namespace)
  namespace.prefix
end

.namespace_uri(namespace) ⇒ Object



542
543
544
# File 'lib/moxml/adapter/ox.rb', line 542

def namespace_uri(namespace)
  namespace.uri
end

.needs_custom_serialize?(node) ⇒ Boolean

Returns:

  • (Boolean)


634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
# File 'lib/moxml/adapter/ox.rb', line 634

def needs_custom_serialize?(node)
  # Fast path: single CData with ]]>
  return true if node.is_a?(::Ox::CData) && node.value&.include?("]]>")

  # Only documents/elements can contain entity refs or CDATA issues
  return false unless node.is_a?(::Ox::Document) || node.is_a?(::Ox::Element)

  # Check cached flags on documents (most common case)
  if node.is_a?(::Ox::Document)
    return true if attachments.get(node, :has_entity_refs)
    return true if attachments.get(node, :has_cdata_end_markers)
    return false if attachments.key?(node, :has_entity_refs) &&
                    attachments.key?(node, :has_cdata_end_markers)
  end

  # Only scan tree on first call — short-circuit on first hit
  has_er = tree_has_entity_references?(node)
  if has_er
    attachments.set(node, :has_entity_refs, true) if node.is_a?(::Ox::Document)
    return true
  end

  has_cdata = tree_has_cdata_end_markers?(node)
  if node.is_a?(::Ox::Document)
    attachments.set(node, :has_entity_refs, false)
    attachments.set(node, :has_cdata_end_markers, has_cdata)
  end

  has_cdata
end

.next_sibling(node) ⇒ Object



267
268
269
270
271
272
273
# File 'lib/moxml/adapter/ox.rb', line 267

def next_sibling(node)
  return unless (parent = node.parent)

  siblings = parent.nodes
  idx = siblings.index(unpatch_node(node))
  idx ? patch_node(siblings[idx + 1], parent) : nil
end

.node_name(node) ⇒ Object



198
199
200
201
202
203
204
205
206
207
# File 'lib/moxml/adapter/ox.rb', line 198

def node_name(node)
  name = begin
    node.value
  rescue StandardError
    node.name
  end

  # Strip namespace prefix if present
  name.to_s.split(":", 2).last
end

.node_type(node) ⇒ Object



182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
# File 'lib/moxml/adapter/ox.rb', line 182

def node_type(node)
  case node
  when ::Ox::Document then :document
  when ::Moxml::Adapter::CustomizedOx::Text, String then :text
  when ::Ox::CData then :cdata
  when ::Ox::Comment then :comment
  when ::Ox::Instruct then :processing_instruction
  when ::Ox::Element then :element
  when ::Ox::DocType then :doctype
  when ::Moxml::Adapter::CustomizedOx::EntityReference then :entity_reference
  when ::Moxml::Adapter::CustomizedOx::Namespace then :banespace
  when ::Moxml::Adapter::CustomizedOx::Attribute then :attribute
  else :unknown
  end
end

.parent(node) ⇒ Object



263
264
265
# File 'lib/moxml/adapter/ox.rb', line 263

def parent(node)
  node.parent if node.is_a?(::Ox::Node)
end

.parse(xml, options = {}, _context = nil) ⇒ Object



22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
# File 'lib/moxml/adapter/ox.rb', line 22

def parse(xml, options = {}, _context = nil)
  processed_xml = preprocess_entities(xml)
  native_doc = begin
    result = ::Ox.parse(processed_xml)

    # result can be either Document or Element
    if result.is_a?(::Ox::Document)
      assign_parents(result)
      validate_single_root(result) if options[:strict]
      result
    else
      doc = ::Ox::Document.new
      doc << result
      assign_parents(doc)
      doc
    end
  rescue ::Ox::ParseError => e
    raise Moxml::ParseError.new(
      e.message,
      source: xml.is_a?(String) ? xml[0..100] : nil,
    )
  end

  ctx = _context || Context.new(:ox)
  Document.new(native_doc, ctx)
end

.patch_node(node, parent = nil) ⇒ Object



220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
# File 'lib/moxml/adapter/ox.rb', line 220

def patch_node(node, parent = nil)
  new_node =
    case node
    # it can be either attribute or namespace
    when Array then ::Moxml::Adapter::CustomizedOx::Attribute.new(
      node.first, node.last
    )
    when Hash then ::Moxml::Adapter::CustomizedOx::Attribute.new(
      node.keys.first, node.values.first
    )
    when String then ::Moxml::Adapter::CustomizedOx::Text.new(node)
    else node
    end

  new_node.parent = parent if new_node.is_a?(::Ox::Node)

  new_node
end

.previous_sibling(node) ⇒ Object



275
276
277
278
279
280
281
# File 'lib/moxml/adapter/ox.rb', line 275

def previous_sibling(node)
  return unless (parent = parent(node))

  siblings = parent.nodes
  idx = siblings.index(unpatch_node(node))
  idx&.positive? ? patch_node(siblings[idx - 1], parent) : nil
end

.processing_instruction_content(node) ⇒ Object



530
531
532
# File 'lib/moxml/adapter/ox.rb', line 530

def processing_instruction_content(node)
  node.content.to_s
end

.processing_instruction_target(node) ⇒ Object



178
179
180
# File 'lib/moxml/adapter/ox.rb', line 178

def processing_instruction_target(node)
  node.target
end

.remove(node) ⇒ Object



423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
# File 'lib/moxml/adapter/ox.rb', line 423

def remove(node)
  return node.clear if node.is_a?(String)

  return unless parent(node)

  # Special handling for declarations on Ox documents
  if parent(node).is_a?(::Ox::Document) && node.is_a?(::Ox::Instruct) && node.target == "xml"
    # Clear declaration attributes from document
    doc = parent(node)
    doc.attributes&.delete(:version)
    doc.attributes&.delete(:encoding)
    doc.attributes&.delete(:standalone)
  end

  parent(node).nodes.delete(unpatch_node(node))
end

.remove_attribute(element, name) ⇒ Object



361
362
363
364
365
366
# File 'lib/moxml/adapter/ox.rb', line 361

def remove_attribute(element, name)
  return unless element.is_a?(::Ox::HasAttrs) && element.attributes

  element.attributes.delete(name.to_s)
  element.attributes.delete(name.to_s.to_sym)
end

.replace(node, new_node) ⇒ Object



440
441
442
443
444
445
446
447
448
449
450
451
452
453
# File 'lib/moxml/adapter/ox.rb', line 440

def replace(node, new_node)
  if node.is_a?(String) && new_node.is_a?(String)
    return node.replace(new_node)
  end
  # There are other cases:
  # when node is a String and new_node isn't
  # when node isn't a String, and new_node is a String

  return unless (parent = parent(node))

  new_node.parent = parent if new_node.is_a?(::Ox::Node)
  idx = parent.nodes.index(node)
  parent.nodes[idx] = new_node if idx
end

.replace_children(node, new_children) ⇒ Object



455
456
457
458
459
460
461
462
# File 'lib/moxml/adapter/ox.rb', line 455

def replace_children(node, new_children)
  node.remove_children_by_path("*")
  new_children.each do |child|
    child.parent = node if child.is_a?(::Ox::Node)
    node << child
  end
  node
end

.root(document) ⇒ Object



289
290
291
# File 'lib/moxml/adapter/ox.rb', line 289

def root(document)
  document.nodes&.find { |node| node.is_a?(::Ox::Element) }
end

.sax_parse(xml, handler) ⇒ void

This method returns an undefined value.

SAX parsing implementation for Ox

Parameters:



54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
# File 'lib/moxml/adapter/ox.rb', line 54

def sax_parse(xml, handler)
  # Create bridge that translates Ox SAX to Moxml SAX
  bridge = OxSAXBridge.new(handler)

  # Parse using Ox's SAX parser
  xml_string = xml.is_a?(IO) || xml.is_a?(StringIO) ? xml.read : xml.to_s

  begin
    ::Ox.sax_parse(bridge, StringIO.new(xml_string))
    # Ox doesn't automatically call end_document, so we do it manually
    bridge.end_document
  rescue ::Ox::ParseError => e
    error = Moxml::ParseError.new(e.message)
    handler.on_error(error)
  end
end

.serialize(node, options = {}) ⇒ Object



624
625
626
627
628
629
630
631
632
# File 'lib/moxml/adapter/ox.rb', line 624

def serialize(node, options = {})
  needs_custom = needs_custom_serialize?(node)

  unless needs_custom
    return serialize_standard(node, options)
  end

  serialize_custom(node, options)
end

.set_attribute(element, name, value) ⇒ Object



310
311
312
313
314
315
316
317
318
319
320
321
322
# File 'lib/moxml/adapter/ox.rb', line 310

def set_attribute(element, name, value)
  element.attributes ||= {}
  if value.nil?
    # Ox converts all values to strings
    remove_attribute(element, name)
  else
    element.attributes[name.to_s] = value
  end

  ::Moxml::Adapter::CustomizedOx::Attribute.new(
    name.to_s, value&.to_s, element
  )
end

.set_attribute_name(attribute, name) ⇒ Object



324
325
326
327
328
329
330
331
# File 'lib/moxml/adapter/ox.rb', line 324

def set_attribute_name(attribute, name)
  old_name = attribute.name
  attribute.name = name.to_s
  # Ox doesn't change the keys of the attributes hash
  element = attribute.parent
  element.attributes.delete(old_name)
  element.attributes[name] = attribute.value
end

.set_attribute_value(attribute, new_value) ⇒ Object



333
334
335
336
337
338
339
340
341
# File 'lib/moxml/adapter/ox.rb', line 333

def set_attribute_value(attribute, new_value)
  if new_value.nil?
    # Ox converts all values to strings
    remove_attribute(attribute.parent, attribute.name)
  else
    attribute.value = new_value
    attribute.parent.attributes[attribute.name] = new_value
  end
end

.set_cdata_content(node, content) ⇒ Object



518
519
520
# File 'lib/moxml/adapter/ox.rb', line 518

def set_cdata_content(node, content)
  node.value = content.to_s
end

.set_comment_content(node, content) ⇒ Object



526
527
528
# File 'lib/moxml/adapter/ox.rb', line 526

def set_comment_content(node, content)
  node.value = content.to_s
end

.set_declaration_attribute(declaration, attr_name, value) ⇒ Object



125
126
127
# File 'lib/moxml/adapter/ox.rb', line 125

def set_declaration_attribute(declaration, attr_name, value)
  set_attribute(declaration, attr_name, value)
end

.set_namespace(element, ns) ⇒ Object



136
137
138
139
140
141
142
143
144
145
146
147
148
# File 'lib/moxml/adapter/ox.rb', line 136

def set_namespace(element, ns)
  return unless element.is_a?(::Ox::Element) || element.is_a?(::Ox::Node)

  prefix = ns.prefix
  # attributes don't have attributes but can have a namespace prefix
  if element.is_a?(::Ox::Element)
    set_attribute(element, ns.expanded_prefix,
                  ns.uri)
  end
  element.name = [prefix,
                  element.name.delete_prefix("xmlns:")].compact.join(":")
  namespace(element)
end

.set_node_name(node, name) ⇒ Object



209
210
211
212
213
214
# File 'lib/moxml/adapter/ox.rb', line 209

def set_node_name(node, name)
  case node
  when ::Ox::Element then node.name = name
  when ::Ox::Instruct then node.value = name
  end
end

.set_processing_instruction_content(node, content) ⇒ Object



534
535
536
# File 'lib/moxml/adapter/ox.rb', line 534

def set_processing_instruction_content(node, content)
  node.content = content.to_s
end

.set_root(doc, element) ⇒ Object



18
19
20
# File 'lib/moxml/adapter/ox.rb', line 18

def set_root(doc, element)
  replace_children(doc, [element])
end

.set_text_content(node, content) ⇒ Object



505
506
507
508
509
510
511
512
# File 'lib/moxml/adapter/ox.rb', line 505

def set_text_content(node, content)
  case node
  when String then node.replace(content.to_s)
  when ::Ox::Element then node.replace_text(content.to_s)
  else
    node.value = content.to_s
  end
end

.text_content(node) ⇒ Object



483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
# File 'lib/moxml/adapter/ox.rb', line 483

def text_content(node)
  return "" if node.nil?

  case node
  when String then node.to_s
  when ::Moxml::Adapter::CustomizedOx::Text then node.value
  when ::Moxml::Adapter::CustomizedOx::EntityReference then ""
  else
    return "" unless node.is_a?(::Ox::Element) || node.is_a?(::Ox::Document)

    node.nodes.map do |n|
      text_content(n)
    end.join
  end
end

.unpatch_node(node) ⇒ Object



239
240
241
242
243
244
245
246
247
248
249
# File 'lib/moxml/adapter/ox.rb', line 239

def unpatch_node(node)
  case node
  # it can be either attribute or namespace
  when ::Moxml::Adapter::CustomizedOx::Attribute then [node.name,
                                                       node.value]
  # when ::Moxml::Adapter::CustomizedOx::Attribute then { node.name => node.value }
  when ::Moxml::Adapter::CustomizedOx::Text then node.value
  when ::Moxml::Adapter::CustomizedOx::EntityReference then node
  else node
  end
end

.validate_single_root(document) ⇒ Object

Raises:



473
474
475
476
477
478
479
480
481
# File 'lib/moxml/adapter/ox.rb', line 473

def validate_single_root(document)
  elements = document.nodes&.grep(::Ox::Element) || []
  return unless elements.size > 1

  raise Moxml::ParseError.new(
    "Multiple root elements found",
    source: nil,
  )
end

.xpath(node, expression, namespaces = {}) ⇒ Object



586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
# File 'lib/moxml/adapter/ox.rb', line 586

def xpath(node, expression, namespaces = {})
  # Translate common XPath patterns to Ox locate() syntax
  locate_expr = translate_xpath_to_locate(expression, namespaces)

  # Ox's locate() works differently on documents vs elements
  # For relative descendant searches on elements, we need special handling
  if expression.start_with?(".//") && node.is_a?(::Ox::Element)
    # Manually search descendants for relative paths from elements
    element_name = locate_expr.sub("?/", "")
    results = []
    traverse(node) do |n|
      next unless n.is_a?(::Ox::Element)

      results << n if n.name == element_name || element_name.empty?
    end
    return results.map do |n|
      patch_node(n, find_parent_in_tree(n, node))
    end
  end

  # Use Ox's locate method for other cases
  results = node.locate(locate_expr)

  # Wrap results and set their parents by finding them in the tree
  results.map { |n| patch_node(n, find_parent_in_tree(n, node)) }
rescue StandardError => e
  raise Moxml::XPathError.new(
    "XPath translation failed: #{e.message}",
    expression: expression,
    adapter: "Ox",
    node: node,
  )
end