Class: Moxml::Adapter::Ox

Inherits:
Base
  • Object
show all
Defined in:
lib/moxml/adapter/ox.rb

Direct Known Subclasses

HeadedOx

Class Method Summary collapse

Methods inherited from Base

actual_native, create_cdata, create_comment, create_declaration, create_doctype, create_element, create_entity_reference, create_namespace, create_processing_instruction, create_text, prepare_for_new_document, sax_supported?

Methods included from XmlUtils

#encode_entities, #normalize_xml_value, #validate_comment_content, #validate_declaration_encoding, #validate_declaration_standalone, #validate_declaration_version, #validate_element_name, #validate_entity_reference_name, #validate_pi_target, #validate_prefix, #validate_uri

Class Method Details

.add_child(element, child) ⇒ Object



367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
# File 'lib/moxml/adapter/ox.rb', line 367

def add_child(element, child)
  # Special handling for declarations on Ox documents
  if element.is_a?(::Ox::Document) && child.is_a?(::Ox::Instruct) && child.target == "xml"
    # Transfer declaration attributes to document
    element.attributes ||= {}
    if child.attributes["version"]
      element.attributes[:version] =
        child.attributes["version"]
    end
    if child.attributes["encoding"]
      element.attributes[:encoding] =
        child.attributes["encoding"]
    end
    if child.attributes["standalone"]
      element.attributes[:standalone] =
        child.attributes["standalone"]
    end
  end

  child.parent = element if child.is_a?(::Ox::Node)
  element.nodes ||= []
  element.nodes << child

  # Mark document if EntityReference is added (avoids tree scan in serialize)
  if child.is_a?(::Moxml::Adapter::CustomizedOx::EntityReference)
    root = element
    while root.is_a?(::Ox::Node) && root.parent
      root = root.parent
    end
    attachments.set(root, :has_entity_refs, true) if root
  end
end

.add_next_sibling(node, sibling) ⇒ Object



411
412
413
414
415
416
417
418
419
420
# File 'lib/moxml/adapter/ox.rb', line 411

def add_next_sibling(node, sibling)
  return unless (parent = parent(node))

  if sibling.is_a?(::Ox::Node)
    sibling.parent&.nodes&.delete(sibling)
    sibling.parent = parent
  end
  idx = parent.nodes.index(node)
  parent.nodes.insert(idx + 1, sibling) if idx
end

.add_previous_sibling(node, sibling) ⇒ Object



400
401
402
403
404
405
406
407
408
409
# File 'lib/moxml/adapter/ox.rb', line 400

def add_previous_sibling(node, sibling)
  return unless (parent = parent(node))

  if sibling.is_a?(::Ox::Node)
    sibling.parent&.nodes&.delete(sibling)
    sibling.parent = parent
  end
  idx = parent.nodes.index(node)
  parent.nodes.insert(idx, sibling) if idx
end

.ancestors(node) ⇒ Object



171
172
173
174
175
# File 'lib/moxml/adapter/ox.rb', line 171

def ancestors(node)
  return [] unless (parent = parent(node))

  [parent] + ancestors(parent)
end

.assign_parents(node, parent = nil) ⇒ Object



463
464
465
466
467
468
469
470
# File 'lib/moxml/adapter/ox.rb', line 463

def assign_parents(node, parent = nil)
  node.parent = parent if node.respond_to?(:parent=) && parent
  return unless node.respond_to?(:nodes)

  node.nodes&.each do |child|
    assign_parents(child, node)
  end
end

.at_xpath(node, expression, namespaces = {}) ⇒ Object



618
619
620
# File 'lib/moxml/adapter/ox.rb', line 618

def at_xpath(node, expression, namespaces = {})
  xpath(node, expression, namespaces)&.first
end

.attachmentsObject



14
15
16
# File 'lib/moxml/adapter/ox.rb', line 14

def attachments
  @attachments ||= Moxml::NativeAttachment.new
end

.attribute_element(attribute) ⇒ Object



305
306
307
# File 'lib/moxml/adapter/ox.rb', line 305

def attribute_element(attribute)
  attribute.parent
end

.attributes(element) ⇒ Object



292
293
294
295
296
297
298
299
300
301
302
303
# File 'lib/moxml/adapter/ox.rb', line 292

def attributes(element)
  return [] unless element.is_a?(::Ox::Element) && element.attributes

  element.attributes.filter_map do |name, value|
    next if name.to_s.start_with?("xmlns")

    # Ensure value is passed correctly - Ox stores with symbol keys
    ::Moxml::Adapter::CustomizedOx::Attribute.new(
      name.to_s, value, element
    )
  end
end

.cdata_content(node) ⇒ Object



513
514
515
# File 'lib/moxml/adapter/ox.rb', line 513

def cdata_content(node)
  node.value.to_s
end

.children(node) ⇒ Object



250
251
252
253
254
255
256
257
258
259
260
# File 'lib/moxml/adapter/ox.rb', line 250

def children(node)
  return [] unless node.is_a?(::Ox::Element) || node.is_a?(::Ox::Document)

  result = node.nodes || []
  # Ox doesn't set parent references during parsing.
  # Set them here so parent/sibling navigation works.
  result.each do |child|
    child.parent = node if child.respond_to?(:parent=)
  end
  result
end

.comment_content(node) ⇒ Object



521
522
523
# File 'lib/moxml/adapter/ox.rb', line 521

def comment_content(node)
  node.value.to_s
end

.create_document(native_doc = nil) ⇒ Object



70
71
72
73
# File 'lib/moxml/adapter/ox.rb', line 70

def create_document(native_doc = nil)
  attrs = native_doc&.attributes || {}
  ::Ox::Document.new(**attrs)
end

.create_native_cdata(content, _owner_doc = nil) ⇒ Object



92
93
94
# File 'lib/moxml/adapter/ox.rb', line 92

def create_native_cdata(content, _owner_doc = nil)
  ::Ox::CData.new(content)
end

.create_native_comment(content, _owner_doc = nil) ⇒ Object



96
97
98
# File 'lib/moxml/adapter/ox.rb', line 96

def create_native_comment(content, _owner_doc = nil)
  ::Ox::Comment.new(content)
end

.create_native_declaration(version, encoding, standalone) ⇒ Object



112
113
114
115
116
117
118
# File 'lib/moxml/adapter/ox.rb', line 112

def create_native_declaration(version, encoding, standalone)
  inst = ::Ox::Instruct.new("xml")
  set_attribute(inst, "version", version)
  set_attribute(inst, "encoding", encoding)
  set_attribute(inst, "standalone", standalone)
  inst
end

.create_native_doctype(name, external_id, system_id) ⇒ Object



100
101
102
103
104
# File 'lib/moxml/adapter/ox.rb', line 100

def create_native_doctype(name, external_id, system_id)
  ::Ox::DocType.new(
    "#{name} PUBLIC \"#{external_id}\" \"#{system_id}\"",
  )
end

.create_native_element(name, _owner_doc = nil) ⇒ Object



75
76
77
78
# File 'lib/moxml/adapter/ox.rb', line 75

def create_native_element(name, _owner_doc = nil)
  element = ::Ox::Element.new(name)
  element
end

.create_native_entity_reference(name) ⇒ Object



84
85
86
# File 'lib/moxml/adapter/ox.rb', line 84

def create_native_entity_reference(name)
  ::Moxml::Adapter::CustomizedOx::EntityReference.new(name)
end

.create_native_namespace(element, prefix, uri) ⇒ Object



128
129
130
131
132
133
# File 'lib/moxml/adapter/ox.rb', line 128

def create_native_namespace(element, prefix, uri)
  ns = ::Moxml::Adapter::CustomizedOx::Namespace.new(prefix, uri,
                                                     element)
  set_attribute(element, ns.expanded_prefix, uri)
  ns
end

.create_native_processing_instruction(target, content) ⇒ Object



106
107
108
109
110
# File 'lib/moxml/adapter/ox.rb', line 106

def create_native_processing_instruction(target, content)
  inst = ::Ox::Instruct.new(target)
  set_processing_instruction_content(inst, content)
  inst
end

.create_native_text(content, _owner_doc = nil) ⇒ Object



80
81
82
# File 'lib/moxml/adapter/ox.rb', line 80

def create_native_text(content, _owner_doc = nil)
  content
end

.declaration_attribute(declaration, attr_name) ⇒ Object



120
121
122
# File 'lib/moxml/adapter/ox.rb', line 120

def declaration_attribute(declaration, attr_name)
  get_attribute_value(declaration, attr_name)
end

.doctype_external_id(native) ⇒ Object



568
569
570
571
572
573
# File 'lib/moxml/adapter/ox.rb', line 568

def doctype_external_id(native)
  value = native.value.to_s
  # Match PUBLIC "external_id"
  match = value.match(/PUBLIC\s+"([^"]*)"/)
  match ? match[1] : nil
end

.doctype_name(native) ⇒ Object

Doctype accessor methods Ox stores DOCTYPE as a string, so we parse it



561
562
563
564
565
566
# File 'lib/moxml/adapter/ox.rb', line 561

def doctype_name(native)
  # Parse: "name PUBLIC \"external_id\" \"system_id\"" or "name SYSTEM \"system_id\""
  value = native.value.to_s.strip
  # Extract the first word (the name)
  value.split(/\s+/).first
end

.doctype_system_id(native) ⇒ Object



575
576
577
578
579
580
581
582
# File 'lib/moxml/adapter/ox.rb', line 575

def doctype_system_id(native)
  value = native.value.to_s
  # Match the last quoted string (system_id)
  # For PUBLIC: "name PUBLIC \"external_id\" \"system_id\""
  # For SYSTEM: "name SYSTEM \"system_id\""
  matches = value.scan(/"([^"]*)"/)
  matches.last&.first
end

.document(node) ⇒ Object



282
283
284
285
286
# File 'lib/moxml/adapter/ox.rb', line 282

def document(node)
  current = node
  current = parent(current) while parent(current)
  current
end

.duplicate_node(node) ⇒ Object



215
216
217
# File 'lib/moxml/adapter/ox.rb', line 215

def duplicate_node(node)
  Marshal.load(Marshal.dump(node))
end

.entity_reference_name(node) ⇒ Object



88
89
90
# File 'lib/moxml/adapter/ox.rb', line 88

def entity_reference_name(node)
  node.name if node.is_a?(::Moxml::Adapter::CustomizedOx::EntityReference)
end

.get_attribute(element, name) ⇒ Object



342
343
344
345
346
347
348
349
350
351
352
353
354
# File 'lib/moxml/adapter/ox.rb', line 342

def get_attribute(element, name)
  return unless element.is_a?(::Ox::HasAttrs) && element.attributes
  unless element.attributes.key?(name.to_s) || element.attributes.key?(name.to_s.to_sym)
    return
  end

  # Ox stores attributes with symbol keys, so try both string and symbol
  value = element.attributes[name.to_s] || element.attributes[name.to_s.to_sym]

  ::Moxml::Adapter::CustomizedOx::Attribute.new(
    name.to_s, value, element
  )
end

.get_attribute_value(element, name) ⇒ Object



356
357
358
# File 'lib/moxml/adapter/ox.rb', line 356

def get_attribute_value(element, name)
  element[name]
end

.has_declaration?(native_doc, _wrapper) ⇒ Boolean

Returns:

  • (Boolean)


636
637
638
639
# File 'lib/moxml/adapter/ox.rb', line 636

def has_declaration?(native_doc, _wrapper)
  # Ox stores declaration in document attributes
  native_doc[:version] || native_doc[:encoding] || native_doc[:standalone]
end

.inner_text(node) ⇒ Object



498
499
500
501
502
# File 'lib/moxml/adapter/ox.rb', line 498

def inner_text(node)
  return "" unless node.is_a?(::Ox::Element) || node.is_a?(::Ox::Document)

  node.nodes.grep(String).join
end

.namespace(element) ⇒ Object



149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
# File 'lib/moxml/adapter/ox.rb', line 149

def namespace(element)
  prefix =
    if element.is_a?(::Moxml::Adapter::CustomizedOx::Attribute)
      element.prefix
    elsif element.name.include?(":")
      element.name.split(":").first
    end
  attr_name = ["xmlns", prefix].compact.join(":")

  ([element] + ancestors(element)).each do |node|
    next unless node.is_a?(::Ox::Element) && node.attributes

    if node[attr_name]
      return ::Moxml::Adapter::CustomizedOx::Namespace.new(
        prefix, node[attr_name], element
      )
    end
  end

  nil
end

.namespace_definitions(node) ⇒ Object



545
546
547
548
549
550
551
552
553
554
555
556
557
# File 'lib/moxml/adapter/ox.rb', line 545

def namespace_definitions(node)
  ([node] + ancestors(node)).reverse.each_with_object({}) do |n, namespaces|
    next unless n.is_a?(::Ox::Element) && n.attributes

    n.attributes.each do |name, value|
      next unless name.to_s.start_with?("xmlns")

      namespaces[name] = ::Moxml::Adapter::CustomizedOx::Namespace.new(
        name, value, n
      )
    end
  end.values
end

.namespace_prefix(namespace) ⇒ Object



537
538
539
# File 'lib/moxml/adapter/ox.rb', line 537

def namespace_prefix(namespace)
  namespace.prefix
end

.namespace_uri(namespace) ⇒ Object



541
542
543
# File 'lib/moxml/adapter/ox.rb', line 541

def namespace_uri(namespace)
  namespace.uri
end

.next_sibling(node) ⇒ Object



266
267
268
269
270
271
272
# File 'lib/moxml/adapter/ox.rb', line 266

def next_sibling(node)
  return unless (parent = node.parent)

  siblings = parent.nodes
  idx = siblings.index(unpatch_node(node))
  idx ? patch_node(siblings[idx + 1], parent) : nil
end

.node_name(node) ⇒ Object



197
198
199
200
201
202
203
204
205
206
# File 'lib/moxml/adapter/ox.rb', line 197

def node_name(node)
  name = begin
    node.value
  rescue StandardError
    node.name
  end

  # Strip namespace prefix if present
  name.to_s.split(":", 2).last
end

.node_type(node) ⇒ Object



181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
# File 'lib/moxml/adapter/ox.rb', line 181

def node_type(node)
  case node
  when ::Ox::Document then :document
  when ::Moxml::Adapter::CustomizedOx::Text, String then :text
  when ::Ox::CData then :cdata
  when ::Ox::Comment then :comment
  when ::Ox::Instruct then :processing_instruction
  when ::Ox::Element then :element
  when ::Ox::DocType then :doctype
  when ::Moxml::Adapter::CustomizedOx::EntityReference then :entity_reference
  when ::Moxml::Adapter::CustomizedOx::Namespace then :banespace
  when ::Moxml::Adapter::CustomizedOx::Attribute then :attribute
  else :unknown
  end
end

.parent(node) ⇒ Object



262
263
264
# File 'lib/moxml/adapter/ox.rb', line 262

def parent(node)
  node.parent if node.is_a?(::Ox::Node)
end

.parse(xml, options = {}, _context = nil) ⇒ Object



22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
# File 'lib/moxml/adapter/ox.rb', line 22

def parse(xml, options = {}, _context = nil)
  native_doc = begin
    result = ::Ox.parse(xml)

    # result can be either Document or Element
    if result.is_a?(::Ox::Document)
      assign_parents(result)
      validate_single_root(result) if options[:strict]
      result
    else
      doc = ::Ox::Document.new
      doc << result
      assign_parents(doc)
      doc
    end
  rescue ::Ox::ParseError => e
    raise Moxml::ParseError.new(
      e.message,
      source: xml.is_a?(String) ? xml[0..100] : nil,
    )
  end

  ctx = _context || Context.new(:ox)
  Document.new(native_doc, ctx)
end

.patch_node(node, parent = nil) ⇒ Object



219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
# File 'lib/moxml/adapter/ox.rb', line 219

def patch_node(node, parent = nil)
  new_node =
    case node
    # it can be either attribute or namespace
    when Array then ::Moxml::Adapter::CustomizedOx::Attribute.new(
      node.first, node.last
    )
    when Hash then ::Moxml::Adapter::CustomizedOx::Attribute.new(
      node.keys.first, node.values.first
    )
    when String then ::Moxml::Adapter::CustomizedOx::Text.new(node)
    else node
    end

  new_node.parent = parent if new_node.is_a?(::Ox::Node)

  new_node
end

.previous_sibling(node) ⇒ Object



274
275
276
277
278
279
280
# File 'lib/moxml/adapter/ox.rb', line 274

def previous_sibling(node)
  return unless (parent = parent(node))

  siblings = parent.nodes
  idx = siblings.index(unpatch_node(node))
  idx&.positive? ? patch_node(siblings[idx - 1], parent) : nil
end

.processing_instruction_content(node) ⇒ Object



529
530
531
# File 'lib/moxml/adapter/ox.rb', line 529

def processing_instruction_content(node)
  node.content.to_s
end

.processing_instruction_target(node) ⇒ Object



177
178
179
# File 'lib/moxml/adapter/ox.rb', line 177

def processing_instruction_target(node)
  node.target
end

.remove(node) ⇒ Object



422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
# File 'lib/moxml/adapter/ox.rb', line 422

def remove(node)
  return node.clear if node.is_a?(String)

  return unless parent(node)

  # Special handling for declarations on Ox documents
  if parent(node).is_a?(::Ox::Document) && node.is_a?(::Ox::Instruct) && node.target == "xml"
    # Clear declaration attributes from document
    doc = parent(node)
    doc.attributes&.delete(:version)
    doc.attributes&.delete(:encoding)
    doc.attributes&.delete(:standalone)
  end

  parent(node).nodes.delete(unpatch_node(node))
end

.remove_attribute(element, name) ⇒ Object



360
361
362
363
364
365
# File 'lib/moxml/adapter/ox.rb', line 360

def remove_attribute(element, name)
  return unless element.is_a?(::Ox::HasAttrs) && element.attributes

  element.attributes.delete(name.to_s)
  element.attributes.delete(name.to_s.to_sym)
end

.replace(node, new_node) ⇒ Object



439
440
441
442
443
444
445
446
447
448
449
450
451
452
# File 'lib/moxml/adapter/ox.rb', line 439

def replace(node, new_node)
  if node.is_a?(String) && new_node.is_a?(String)
    return node.replace(new_node)
  end
  # There are other cases:
  # when node is a String and new_node isn't
  # when node isn't a String, and new_node is a String

  return unless (parent = parent(node))

  new_node.parent = parent if new_node.is_a?(::Ox::Node)
  idx = parent.nodes.index(node)
  parent.nodes[idx] = new_node if idx
end

.replace_children(node, new_children) ⇒ Object



454
455
456
457
458
459
460
461
# File 'lib/moxml/adapter/ox.rb', line 454

def replace_children(node, new_children)
  node.remove_children_by_path("*")
  new_children.each do |child|
    child.parent = node if child.is_a?(::Ox::Node)
    node << child
  end
  node
end

.root(document) ⇒ Object



288
289
290
# File 'lib/moxml/adapter/ox.rb', line 288

def root(document)
  document.nodes&.find { |node| node.is_a?(::Ox::Element) }
end

.sax_parse(xml, handler) ⇒ void

This method returns an undefined value.

SAX parsing implementation for Ox

Parameters:



53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
# File 'lib/moxml/adapter/ox.rb', line 53

def sax_parse(xml, handler)
  # Create bridge that translates Ox SAX to Moxml SAX
  bridge = OxSAXBridge.new(handler)

  # Parse using Ox's SAX parser
  xml_string = xml.is_a?(IO) || xml.is_a?(StringIO) ? xml.read : xml.to_s

  begin
    ::Ox.sax_parse(bridge, StringIO.new(xml_string))
    # Ox doesn't automatically call end_document, so we do it manually
    bridge.end_document
  rescue ::Ox::ParseError => e
    error = Moxml::ParseError.new(e.message)
    handler.on_error(error)
  end
end

.serialize(node, options = {}) ⇒ Object



622
623
624
625
626
627
628
629
630
631
632
633
634
# File 'lib/moxml/adapter/ox.rb', line 622

def serialize(node, options = {})
  # Fast path: skip EntityReference scan for documents (most common case)
  if node.is_a?(::Ox::Document) &&
      !attachments.get(node, :has_entity_refs)
    return serialize_standard(node, options)
  end

  if tree_has_entity_references?(node)
    serialize_custom(node, options)
  else
    serialize_standard(node, options)
  end
end

.set_attribute(element, name, value) ⇒ Object



309
310
311
312
313
314
315
316
317
318
319
320
321
# File 'lib/moxml/adapter/ox.rb', line 309

def set_attribute(element, name, value)
  element.attributes ||= {}
  if value.nil?
    # Ox converts all values to strings
    remove_attribute(element, name)
  else
    element.attributes[name.to_s] = value
  end

  ::Moxml::Adapter::CustomizedOx::Attribute.new(
    name.to_s, value&.to_s, element
  )
end

.set_attribute_name(attribute, name) ⇒ Object



323
324
325
326
327
328
329
330
# File 'lib/moxml/adapter/ox.rb', line 323

def set_attribute_name(attribute, name)
  old_name = attribute.name
  attribute.name = name.to_s
  # Ox doesn't change the keys of the attributes hash
  element = attribute.parent
  element.attributes.delete(old_name)
  element.attributes[name] = attribute.value
end

.set_attribute_value(attribute, new_value) ⇒ Object



332
333
334
335
336
337
338
339
340
# File 'lib/moxml/adapter/ox.rb', line 332

def set_attribute_value(attribute, new_value)
  if new_value.nil?
    # Ox converts all values to strings
    remove_attribute(attribute.parent, attribute.name)
  else
    attribute.value = new_value
    attribute.parent.attributes[attribute.name] = new_value
  end
end

.set_cdata_content(node, content) ⇒ Object



517
518
519
# File 'lib/moxml/adapter/ox.rb', line 517

def set_cdata_content(node, content)
  node.value = content.to_s
end

.set_comment_content(node, content) ⇒ Object



525
526
527
# File 'lib/moxml/adapter/ox.rb', line 525

def set_comment_content(node, content)
  node.value = content.to_s
end

.set_declaration_attribute(declaration, attr_name, value) ⇒ Object



124
125
126
# File 'lib/moxml/adapter/ox.rb', line 124

def set_declaration_attribute(declaration, attr_name, value)
  set_attribute(declaration, attr_name, value)
end

.set_namespace(element, ns) ⇒ Object



135
136
137
138
139
140
141
142
143
144
145
146
147
# File 'lib/moxml/adapter/ox.rb', line 135

def set_namespace(element, ns)
  return unless element.is_a?(::Ox::Element) || element.is_a?(::Ox::Node)

  prefix = ns.prefix
  # attributes don't have attributes but can have a namespace prefix
  if element.is_a?(::Ox::Element)
    set_attribute(element, ns.expanded_prefix,
                  ns.uri)
  end
  element.name = [prefix,
                  element.name.delete_prefix("xmlns:")].compact.join(":")
  namespace(element)
end

.set_node_name(node, name) ⇒ Object



208
209
210
211
212
213
# File 'lib/moxml/adapter/ox.rb', line 208

def set_node_name(node, name)
  case node
  when ::Ox::Element then node.name = name
  when ::Ox::Instruct then node.value = name
  end
end

.set_processing_instruction_content(node, content) ⇒ Object



533
534
535
# File 'lib/moxml/adapter/ox.rb', line 533

def set_processing_instruction_content(node, content)
  node.content = content.to_s
end

.set_root(doc, element) ⇒ Object



18
19
20
# File 'lib/moxml/adapter/ox.rb', line 18

def set_root(doc, element)
  replace_children(doc, [element])
end

.set_text_content(node, content) ⇒ Object



504
505
506
507
508
509
510
511
# File 'lib/moxml/adapter/ox.rb', line 504

def set_text_content(node, content)
  case node
  when String then node.replace(content.to_s)
  when ::Ox::Element then node.replace_text(content.to_s)
  else
    node.value = content.to_s
  end
end

.text_content(node) ⇒ Object



482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
# File 'lib/moxml/adapter/ox.rb', line 482

def text_content(node)
  return "" if node.nil?

  case node
  when String then node.to_s
  when ::Moxml::Adapter::CustomizedOx::Text then node.value
  when ::Moxml::Adapter::CustomizedOx::EntityReference then ""
  else
    return "" unless node.is_a?(::Ox::Element) || node.is_a?(::Ox::Document)

    node.nodes.map do |n|
      text_content(n)
    end.join
  end
end

.unpatch_node(node) ⇒ Object



238
239
240
241
242
243
244
245
246
247
248
# File 'lib/moxml/adapter/ox.rb', line 238

def unpatch_node(node)
  case node
  # it can be either attribute or namespace
  when ::Moxml::Adapter::CustomizedOx::Attribute then [node.name,
                                                       node.value]
  # when ::Moxml::Adapter::CustomizedOx::Attribute then { node.name => node.value }
  when ::Moxml::Adapter::CustomizedOx::Text then node.value
  when ::Moxml::Adapter::CustomizedOx::EntityReference then node
  else node
  end
end

.validate_single_root(document) ⇒ Object

Raises:



472
473
474
475
476
477
478
479
480
# File 'lib/moxml/adapter/ox.rb', line 472

def validate_single_root(document)
  elements = document.nodes&.grep(::Ox::Element) || []
  return unless elements.size > 1

  raise Moxml::ParseError.new(
    "Multiple root elements found",
    source: nil,
  )
end

.xpath(node, expression, namespaces = {}) ⇒ Object



584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
# File 'lib/moxml/adapter/ox.rb', line 584

def xpath(node, expression, namespaces = {})
  # Translate common XPath patterns to Ox locate() syntax
  locate_expr = translate_xpath_to_locate(expression, namespaces)

  # Ox's locate() works differently on documents vs elements
  # For relative descendant searches on elements, we need special handling
  if expression.start_with?(".//") && node.is_a?(::Ox::Element)
    # Manually search descendants for relative paths from elements
    element_name = locate_expr.sub("?/", "")
    results = []
    traverse(node) do |n|
      next unless n.is_a?(::Ox::Element)

      results << n if n.name == element_name || element_name.empty?
    end
    return results.map do |n|
      patch_node(n, find_parent_in_tree(n, node))
    end
  end

  # Use Ox's locate method for other cases
  results = node.locate(locate_expr)

  # Wrap results and set their parents by finding them in the tree
  results.map { |n| patch_node(n, find_parent_in_tree(n, node)) }
rescue StandardError => e
  raise Moxml::XPathError.new(
    "XPath translation failed: #{e.message}",
    expression: expression,
    adapter: "Ox",
    node: node,
  )
end