Class: Moxml::Adapter::Ox

Inherits:
Base
  • Object
show all
Defined in:
lib/moxml/adapter/ox.rb

Direct Known Subclasses

HeadedOx

Class Method Summary collapse

Methods inherited from Base

create_cdata, create_comment, create_declaration, create_doctype, create_element, create_namespace, create_processing_instruction, create_text, prepare_for_new_document, sax_supported?

Methods included from XmlUtils

#encode_entities, #normalize_xml_value, #validate_comment_content, #validate_declaration_encoding, #validate_declaration_standalone, #validate_declaration_version, #validate_element_name, #validate_pi_target, #validate_prefix, #validate_uri

Class Method Details

.add_child(element, child) ⇒ Object



350
351
352
353
354
# File 'lib/moxml/adapter/ox.rb', line 350

def add_child(element, child)
  child.parent = element if child.respond_to?(:parent)
  element.nodes ||= []
  element.nodes << child
end

.add_next_sibling(node, sibling) ⇒ Object



367
368
369
370
371
372
373
374
375
376
# File 'lib/moxml/adapter/ox.rb', line 367

def add_next_sibling(node, sibling)
  return unless (parent = parent(node))

  if sibling.respond_to?(:parent)
    sibling.parent&.nodes&.delete(sibling)
    sibling.parent = parent
  end
  idx = parent.nodes.index(node)
  parent.nodes.insert(idx + 1, sibling) if idx
end

.add_previous_sibling(node, sibling) ⇒ Object



356
357
358
359
360
361
362
363
364
365
# File 'lib/moxml/adapter/ox.rb', line 356

def add_previous_sibling(node, sibling)
  return unless (parent = parent(node))

  if sibling.respond_to?(:parent)
    sibling.parent&.nodes&.delete(sibling)
    sibling.parent = parent
  end
  idx = parent.nodes.index(node)
  parent.nodes.insert(idx, sibling) if idx
end

.ancestors(node) ⇒ Object



159
160
161
162
163
# File 'lib/moxml/adapter/ox.rb', line 159

def ancestors(node)
  return [] unless (parent = parent(node))

  [parent] + ancestors(parent)
end

.at_xpath(node, expression, namespaces = {}) ⇒ Object



520
521
522
# File 'lib/moxml/adapter/ox.rb', line 520

def at_xpath(node, expression, namespaces = {})
  xpath(node, expression, namespaces)&.first
end

.attribute_element(attribute) ⇒ Object



288
289
290
# File 'lib/moxml/adapter/ox.rb', line 288

def attribute_element(attribute)
  attribute.parent
end

.attributes(element) ⇒ Object



273
274
275
276
277
278
279
280
281
282
283
284
285
286
# File 'lib/moxml/adapter/ox.rb', line 273

def attributes(element)
  unless element.respond_to?(:attributes) && element.attributes
    return []
  end

  element.attributes.filter_map do |name, value|
    next if name.to_s.start_with?("xmlns")

    # Ensure value is passed correctly - Ox stores with symbol keys
    ::Moxml::Adapter::CustomizedOx::Attribute.new(
      name.to_s, value, element
    )
  end
end

.cdata_content(node) ⇒ Object



440
441
442
# File 'lib/moxml/adapter/ox.rb', line 440

def cdata_content(node)
  node.value.to_s
end

.children(node) ⇒ Object



237
238
239
240
241
# File 'lib/moxml/adapter/ox.rb', line 237

def children(node)
  return [] unless node.respond_to?(:nodes)

  node.nodes || []
end

.comment_content(node) ⇒ Object



448
449
450
# File 'lib/moxml/adapter/ox.rb', line 448

def comment_content(node)
  node.value.to_s
end

.create_document(native_doc = nil) ⇒ Object



64
65
66
67
# File 'lib/moxml/adapter/ox.rb', line 64

def create_document(native_doc = nil)
  attrs = native_doc&.attributes || {}
  ::Ox::Document.new(**attrs)
end

.create_native_cdata(content) ⇒ Object



79
80
81
# File 'lib/moxml/adapter/ox.rb', line 79

def create_native_cdata(content)
  ::Ox::CData.new(content)
end

.create_native_comment(content) ⇒ Object



83
84
85
# File 'lib/moxml/adapter/ox.rb', line 83

def create_native_comment(content)
  ::Ox::Comment.new(content)
end

.create_native_declaration(version, encoding, standalone) ⇒ Object



99
100
101
102
103
104
105
# File 'lib/moxml/adapter/ox.rb', line 99

def create_native_declaration(version, encoding, standalone)
  inst = ::Ox::Instruct.new("xml")
  set_attribute(inst, "version", version)
  set_attribute(inst, "encoding", encoding)
  set_attribute(inst, "standalone", standalone)
  inst
end

.create_native_doctype(name, external_id, system_id) ⇒ Object



87
88
89
90
91
# File 'lib/moxml/adapter/ox.rb', line 87

def create_native_doctype(name, external_id, system_id)
  ::Ox::DocType.new(
    "#{name} PUBLIC \"#{external_id}\" \"#{system_id}\"",
  )
end

.create_native_element(name) ⇒ Object



69
70
71
72
73
# File 'lib/moxml/adapter/ox.rb', line 69

def create_native_element(name)
  element = ::Ox::Element.new(name)
  element.instance_variable_set(:@attributes, {})
  element
end

.create_native_namespace(element, prefix, uri) ⇒ Object



115
116
117
118
119
120
# File 'lib/moxml/adapter/ox.rb', line 115

def create_native_namespace(element, prefix, uri)
  ns = ::Moxml::Adapter::CustomizedOx::Namespace.new(prefix, uri,
                                                     element)
  set_attribute(element, ns.expanded_prefix, uri)
  ns
end

.create_native_processing_instruction(target, content) ⇒ Object



93
94
95
96
97
# File 'lib/moxml/adapter/ox.rb', line 93

def create_native_processing_instruction(target, content)
  inst = ::Ox::Instruct.new(target)
  set_processing_instruction_content(inst, content)
  inst
end

.create_native_text(content) ⇒ Object



75
76
77
# File 'lib/moxml/adapter/ox.rb', line 75

def create_native_text(content)
  content
end

.declaration_attribute(declaration, attr_name) ⇒ Object



107
108
109
# File 'lib/moxml/adapter/ox.rb', line 107

def declaration_attribute(declaration, attr_name)
  get_attribute_value(declaration, attr_name)
end

.document(node) ⇒ Object



263
264
265
266
267
# File 'lib/moxml/adapter/ox.rb', line 263

def document(node)
  current = node
  current = parent(current) while parent(current)
  current
end

.duplicate_node(node) ⇒ Object



203
204
205
# File 'lib/moxml/adapter/ox.rb', line 203

def duplicate_node(node)
  Marshal.load(Marshal.dump(node))
end

.get_attribute(element, name) ⇒ Object



325
326
327
328
329
330
331
332
333
334
335
336
337
# File 'lib/moxml/adapter/ox.rb', line 325

def get_attribute(element, name)
  return unless element.respond_to?(:attributes) && element.attributes
  unless element.attributes.key?(name.to_s) || element.attributes.key?(name.to_s.to_sym)
    return
  end

  # Ox stores attributes with symbol keys, so try both string and symbol
  value = element.attributes[name.to_s] || element.attributes[name.to_s.to_sym]

  ::Moxml::Adapter::CustomizedOx::Attribute.new(
    name.to_s, value, element
  )
end

.get_attribute_value(element, name) ⇒ Object



339
340
341
# File 'lib/moxml/adapter/ox.rb', line 339

def get_attribute_value(element, name)
  element[name]
end

.inner_text(node) ⇒ Object



425
426
427
428
429
# File 'lib/moxml/adapter/ox.rb', line 425

def inner_text(node)
  return "" unless node.respond_to?(:nodes)

  node.nodes.select { _1.is_a?(String) }.join
end

.namespace(element) ⇒ Object



136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
# File 'lib/moxml/adapter/ox.rb', line 136

def namespace(element)
  prefix =
    if element.respond_to?(:prefix)
      # attribute
      element.prefix
    elsif element.name.include?(":")
      element.name.split(":").first
    end
  attr_name = ["xmlns", prefix].compact.join(":")

  ([element] + ancestors(element)).each do |node|
    next unless node.respond_to?(:attributes) && node.attributes

    if node[attr_name]
      return ::Moxml::Adapter::CustomizedOx::Namespace.new(
        prefix, node[attr_name], element
      )
    end
  end

  nil
end

.namespace_definitions(node) ⇒ Object



472
473
474
475
476
477
478
479
480
481
482
483
484
# File 'lib/moxml/adapter/ox.rb', line 472

def namespace_definitions(node)
  ([node] + ancestors(node)).reverse.each_with_object({}) do |n, namespaces|
    next unless n.respond_to?(:attributes) && n.attributes

    n.attributes.each do |name, value|
      next unless name.to_s.start_with?("xmlns")

      namespaces[name] = ::Moxml::Adapter::CustomizedOx::Namespace.new(
        name, value, n
      )
    end
  end.values
end

.namespace_prefix(namespace) ⇒ Object



464
465
466
# File 'lib/moxml/adapter/ox.rb', line 464

def namespace_prefix(namespace)
  namespace.prefix
end

.namespace_uri(namespace) ⇒ Object



468
469
470
# File 'lib/moxml/adapter/ox.rb', line 468

def namespace_uri(namespace)
  namespace.uri
end

.next_sibling(node) ⇒ Object



247
248
249
250
251
252
253
# File 'lib/moxml/adapter/ox.rb', line 247

def next_sibling(node)
  return unless (parent = node.parent)

  siblings = parent.nodes
  idx = siblings.index(unpatch_node(node))
  idx ? patch_node(siblings[idx + 1], parent) : nil
end

.node_name(node) ⇒ Object



184
185
186
187
188
189
190
191
192
193
# File 'lib/moxml/adapter/ox.rb', line 184

def node_name(node)
  name = begin
    node.value
  rescue StandardError
    node.name
  end

  # Strip namespace prefix if present
  name.to_s.split(":", 2).last
end

.node_type(node) ⇒ Object



169
170
171
172
173
174
175
176
177
178
179
180
181
182
# File 'lib/moxml/adapter/ox.rb', line 169

def node_type(node)
  case node
  when ::Ox::Document then :document
  when ::Moxml::Adapter::CustomizedOx::Text, String then :text
  when ::Ox::CData then :cdata
  when ::Ox::Comment then :comment
  when ::Ox::Instruct then :processing_instruction
  when ::Ox::Element then :element
  when ::Ox::DocType then :doctype
  when ::Moxml::Adapter::CustomizedOx::Namespace then :banespace
  when ::Moxml::Adapter::CustomizedOx::Attribute then :attribute
  else :unknown
  end
end

.parent(node) ⇒ Object



243
244
245
# File 'lib/moxml/adapter/ox.rb', line 243

def parent(node)
  node.parent if node.respond_to?(:parent)
end

.parse(xml, _options = {}) ⇒ Object



20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
# File 'lib/moxml/adapter/ox.rb', line 20

def parse(xml, _options = {})
  native_doc = begin
    result = ::Ox.parse(xml)

    # result can be either Document or Element
    if result.is_a?(::Ox::Document)
      result
    else
      doc = ::Ox::Document.new
      doc << result
      doc
    end
  rescue ::Ox::ParseError => e
    raise Moxml::ParseError.new(
      e.message,
      source: xml.is_a?(String) ? xml[0..100] : nil,
    )
  end

  DocumentBuilder.new(Context.new(:ox)).build(native_doc)
end

.patch_node(node, parent = nil) ⇒ Object



207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
# File 'lib/moxml/adapter/ox.rb', line 207

def patch_node(node, parent = nil)
  new_node =
    case node
    # it can be either attribute or namespace
    when Array then ::Moxml::Adapter::CustomizedOx::Attribute.new(
      node.first, node.last
    )
    when Hash then ::Moxml::Adapter::CustomizedOx::Attribute.new(
      node.keys.first, node.values.first
    )
    when String then ::Moxml::Adapter::CustomizedOx::Text.new(node)
    else node
    end

  new_node.parent = parent if new_node.respond_to?(:parent)

  new_node
end

.previous_sibling(node) ⇒ Object



255
256
257
258
259
260
261
# File 'lib/moxml/adapter/ox.rb', line 255

def previous_sibling(node)
  return unless (parent = parent(node))

  siblings = parent.nodes
  idx = siblings.index(unpatch_node(node))
  idx&.positive? ? patch_node(siblings[idx - 1], parent) : nil
end

.processing_instruction_content(node) ⇒ Object



456
457
458
# File 'lib/moxml/adapter/ox.rb', line 456

def processing_instruction_content(node)
  node.content.to_s
end

.processing_instruction_target(node) ⇒ Object



165
166
167
# File 'lib/moxml/adapter/ox.rb', line 165

def processing_instruction_target(node)
  node.target
end

.remove(node) ⇒ Object



378
379
380
381
382
383
384
# File 'lib/moxml/adapter/ox.rb', line 378

def remove(node)
  return node.clear if node.is_a?(String)

  return unless parent(node)

  parent(node).nodes.delete(unpatch_node(node))
end

.remove_attribute(element, name) ⇒ Object



343
344
345
346
347
348
# File 'lib/moxml/adapter/ox.rb', line 343

def remove_attribute(element, name)
  return unless element.respond_to?(:attributes) && element.attributes

  element.attributes.delete(name.to_s)
  element.attributes.delete(name.to_s.to_sym)
end

.replace(node, new_node) ⇒ Object



386
387
388
389
390
391
392
393
394
395
396
397
398
399
# File 'lib/moxml/adapter/ox.rb', line 386

def replace(node, new_node)
  if node.is_a?(String) && new_node.is_a?(String)
    return node.replace(new_node)
  end
  # There are other cases:
  # when node is a String and new_node isn't
  # when node isn't a String, and new_node is a String

  return unless (parent = parent(node))

  new_node.parent = parent if new_node.respond_to?(:parent)
  idx = parent.nodes.index(node)
  parent.nodes[idx] = new_node if idx
end

.replace_children(node, new_children) ⇒ Object



401
402
403
404
405
406
407
408
# File 'lib/moxml/adapter/ox.rb', line 401

def replace_children(node, new_children)
  node.remove_children_by_path("*")
  new_children.each do |child|
    child.parent = node if child.respond_to?(:parent)
    node << child
  end
  node
end

.root(document) ⇒ Object



269
270
271
# File 'lib/moxml/adapter/ox.rb', line 269

def root(document)
  document.nodes&.find { |node| node.is_a?(::Ox::Element) }
end

.sax_parse(xml, handler) ⇒ void

This method returns an undefined value.

SAX parsing implementation for Ox

Parameters:



47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
# File 'lib/moxml/adapter/ox.rb', line 47

def sax_parse(xml, handler)
  # Create bridge that translates Ox SAX to Moxml SAX
  bridge = OxSAXBridge.new(handler)

  # Parse using Ox's SAX parser
  xml_string = xml.respond_to?(:read) ? xml.read : xml.to_s

  begin
    ::Ox.sax_parse(bridge, StringIO.new(xml_string))
    # Ox doesn't automatically call end_document, so we do it manually
    bridge.end_document
  rescue ::Ox::ParseError => e
    error = Moxml::ParseError.new(e.message)
    handler.on_error(error)
  end
end

.serialize(node, options = {}) ⇒ Object



524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
# File 'lib/moxml/adapter/ox.rb', line 524

def serialize(node, options = {})
  output = ""
  if node.is_a?(::Ox::Document)
    # add declaration
    version = node[:version] || "1.0"
    encoding = options[:encoding] || node[:encoding]
    standalone = node[:standalone]

    decl = create_native_declaration(version, encoding, standalone)
    output = ::Ox.dump(::Ox::Document.new << decl).strip
  end

  ox_options = {
    indent: -1, # options[:indent] || -1, # indent is a beast
    # with_xml: true,
    with_instructions: true,
    encoding: options[:encoding],
    no_empty: options[:expand_empty],
  }
  output + ::Ox.dump(node, ox_options)
end

.set_attribute(element, name, value) ⇒ Object



292
293
294
295
296
297
298
299
300
301
302
303
304
# File 'lib/moxml/adapter/ox.rb', line 292

def set_attribute(element, name, value)
  element.attributes ||= {}
  if value.nil?
    # Ox converts all values to strings
    remove_attribute(element, name)
  else
    element.attributes[name.to_s] = value
  end

  ::Moxml::Adapter::CustomizedOx::Attribute.new(
    name.to_s, value&.to_s, element
  )
end

.set_attribute_name(attribute, name) ⇒ Object



306
307
308
309
310
311
312
313
# File 'lib/moxml/adapter/ox.rb', line 306

def set_attribute_name(attribute, name)
  old_name = attribute.name
  attribute.name = name.to_s
  # Ox doesn't change the keys of the attributes hash
  element = attribute.parent
  element.attributes.delete(old_name)
  element.attributes[name] = attribute.value
end

.set_attribute_value(attribute, new_value) ⇒ Object



315
316
317
318
319
320
321
322
323
# File 'lib/moxml/adapter/ox.rb', line 315

def set_attribute_value(attribute, new_value)
  if new_value.nil?
    # Ox converts all values to strings
    remove_attribute(attribute.parent, attribute.name)
  else
    attribute.value = new_value
    attribute.parent.attributes[attribute.name] = new_value
  end
end

.set_cdata_content(node, content) ⇒ Object



444
445
446
# File 'lib/moxml/adapter/ox.rb', line 444

def set_cdata_content(node, content)
  node.value = content.to_s
end

.set_comment_content(node, content) ⇒ Object



452
453
454
# File 'lib/moxml/adapter/ox.rb', line 452

def set_comment_content(node, content)
  node.value = content.to_s
end

.set_declaration_attribute(declaration, attr_name, value) ⇒ Object



111
112
113
# File 'lib/moxml/adapter/ox.rb', line 111

def set_declaration_attribute(declaration, attr_name, value)
  set_attribute(declaration, attr_name, value)
end

.set_namespace(element, ns) ⇒ Object



122
123
124
125
126
127
128
129
130
131
132
133
134
# File 'lib/moxml/adapter/ox.rb', line 122

def set_namespace(element, ns)
  return unless element.respond_to?(:name)

  prefix = ns.prefix
  # attributes don't have attributes but can have a namespace prefix
  if element.respond_to?(:attributes)
    set_attribute(element, ns.expanded_prefix,
                  ns.uri)
  end
  element.name = [prefix,
                  element.name.delete_prefix("xmlns:")].compact.join(":")
  namespace(element)
end

.set_node_name(node, name) ⇒ Object



195
196
197
198
199
200
201
# File 'lib/moxml/adapter/ox.rb', line 195

def set_node_name(node, name)
  if node.respond_to?(:name=)
    node.name = name
  elsif node.respond_to?(:value=)
    node.value = name
  end
end

.set_processing_instruction_content(node, content) ⇒ Object



460
461
462
# File 'lib/moxml/adapter/ox.rb', line 460

def set_processing_instruction_content(node, content)
  node.content = content.to_s
end

.set_root(doc, element) ⇒ Object



16
17
18
# File 'lib/moxml/adapter/ox.rb', line 16

def set_root(doc, element)
  replace_children(doc, [element])
end

.set_text_content(node, content) ⇒ Object



431
432
433
434
435
436
437
438
# File 'lib/moxml/adapter/ox.rb', line 431

def set_text_content(node, content)
  case node
  when String then node.replace(content.to_s)
  when ::Ox::Element then node.replace_text(content.to_s)
  else
    node.value = content.to_s
  end
end

.text_content(node) ⇒ Object



410
411
412
413
414
415
416
417
418
419
420
421
422
423
# File 'lib/moxml/adapter/ox.rb', line 410

def text_content(node)
  return "" if node.nil?

  case node
  when String then node.to_s
  when ::Moxml::Adapter::CustomizedOx::Text then node.value
  else
    return "" unless node.respond_to?(:nodes)

    node.nodes.map do |n|
      text_content(n)
    end.join
  end
end

.unpatch_node(node) ⇒ Object



226
227
228
229
230
231
232
233
234
235
# File 'lib/moxml/adapter/ox.rb', line 226

def unpatch_node(node)
  case node
  # it can be either attribute or namespace
  when ::Moxml::Adapter::CustomizedOx::Attribute then [node.name,
                                                       node.value]
  # when ::Moxml::Adapter::CustomizedOx::Attribute then { node.name => node.value }
  when ::Moxml::Adapter::CustomizedOx::Text then node.value
  else node
  end
end

.xpath(node, expression, namespaces = {}) ⇒ Object



486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
# File 'lib/moxml/adapter/ox.rb', line 486

def xpath(node, expression, namespaces = {})
  # Translate common XPath patterns to Ox locate() syntax
  locate_expr = translate_xpath_to_locate(expression, namespaces)

  # Ox's locate() works differently on documents vs elements
  # For relative descendant searches on elements, we need special handling
  if expression.start_with?(".//") && node.is_a?(::Ox::Element)
    # Manually search descendants for relative paths from elements
    element_name = locate_expr.sub("?/", "")
    results = []
    traverse(node) do |n|
      next unless n.is_a?(::Ox::Element)

      results << n if n.name == element_name || element_name.empty?
    end
    return results.map do |n|
      patch_node(n, find_parent_in_tree(n, node))
    end
  end

  # Use Ox's locate method for other cases
  results = node.locate(locate_expr)

  # Wrap results and set their parents by finding them in the tree
  results.map { |n| patch_node(n, find_parent_in_tree(n, node)) }
rescue StandardError => e
  raise Moxml::XPathError.new(
    "XPath translation failed: #{e.message}",
    expression: expression,
    adapter: "Ox",
    node: node,
  )
end