Class: Moxml::Adapter::Rexml
- Inherits:
-
Base
- Object
- Base
- Moxml::Adapter::Rexml
show all
- Defined in:
- lib/moxml/adapter/rexml.rb
Constant Summary
Constants inherited
from Base
Base::ENTITY_MARKER, Base::ENTITY_MARKER_RE, Base::ENTITY_NAME_PATTERN, Base::ENTITY_NAME_RE, Base::SERIALIZED_ENTITY_MARKER_RE, Base::STANDARD_ENTITIES
Class Method Summary
collapse
-
.add_child(element, child) ⇒ Object
-
.add_next_sibling(node, sibling) ⇒ Object
-
.add_previous_sibling(node, sibling) ⇒ Object
-
.append_child_sequence(element, type) ⇒ Object
-
.at_xpath(node, expression, namespaces = {}) ⇒ Object
-
.attachments ⇒ Object
-
.attribute_element(attribute) ⇒ Object
-
.attributes(element) ⇒ Object
-
.cdata_content(node) ⇒ Object
-
.children(node) ⇒ Object
-
.comment_content(node) ⇒ Object
-
.create_document(_native_doc = nil) ⇒ Object
-
.create_native_cdata(content, _owner_doc = nil) ⇒ Object
-
.create_native_comment(content, _owner_doc = nil) ⇒ Object
-
.create_native_declaration(version, encoding, standalone) ⇒ Object
-
.create_native_doctype(name, external_id, system_id) ⇒ Object
-
.create_native_element(name, _owner_doc = nil) ⇒ Object
-
.create_native_entity_reference(name) ⇒ Object
-
.create_native_namespace(element, prefix, uri) ⇒ Object
add a namespace definition, keep the element name unchanged.
-
.create_native_processing_instruction(target, content) ⇒ Object
-
.create_native_text(content, _owner_doc = nil) ⇒ Object
-
.declaration_attribute(node, name) ⇒ Object
-
.doctype_external_id(native) ⇒ Object
-
.doctype_name(native) ⇒ Object
Doctype accessor methods.
-
.doctype_system_id(native) ⇒ Object
-
.document(node) ⇒ Object
-
.duplicate_node(node) ⇒ Object
-
.entity_reference_name(node) ⇒ Object
-
.extract_encoding_from_xml(xml) ⇒ Object
-
.extract_text_recursively(element) ⇒ Object
-
.get_attribute(element, name) ⇒ Object
-
.get_attribute_value(element, name) ⇒ Object
-
.has_declaration?(native_doc, wrapper) ⇒ Boolean
-
.in_scope_namespaces(element) ⇒ Object
-
.inner_text(node) ⇒ Object
-
.namespace(node) ⇒ Object
-
.namespace_definitions(node) ⇒ Object
-
.namespace_prefix(node) ⇒ Object
-
.namespace_uri(node) ⇒ Object
-
.next_sibling(node) ⇒ Object
-
.node_name(node) ⇒ Object
-
.node_type(node) ⇒ Object
-
.parent(node) ⇒ Object
-
.parse(xml, options = {}, _context = nil) ⇒ Object
-
.prepare_xpath_namespaces(node) ⇒ Object
not used at the moment but may be useful when the xpath is upgraded to work with namespaces.
-
.previous_sibling(node) ⇒ Object
-
.processing_instruction_content(node) ⇒ Object
-
.processing_instruction_target(node) ⇒ Object
-
.remove(node) ⇒ Object
-
.remove_attribute(element, name) ⇒ Object
-
.replace(node, new_node) ⇒ Object
-
.replace_children(element, children) ⇒ Object
-
.root(document) ⇒ Object
-
.sax_parse(xml, handler) ⇒ void
SAX parsing implementation for REXML.
-
.serialize(node, options = {}) ⇒ Object
-
.set_attribute(element, name, value) ⇒ Object
-
.set_attribute_name(attribute, name) ⇒ Object
-
.set_attribute_value(attribute, value) ⇒ Object
-
.set_cdata_content(node, content) ⇒ Object
-
.set_comment_content(node, content) ⇒ Object
-
.set_declaration_attribute(node, name, value) ⇒ Object
-
.set_namespace(element, ns) ⇒ Object
add a namespace prefix to the element name AND a namespace definition.
-
.set_node_name(node, name) ⇒ Object
-
.set_processing_instruction_content(node, content) ⇒ Object
-
.set_root(doc, element) ⇒ Object
-
.set_text_content(node, content) ⇒ Object
-
.text_content(node) ⇒ Object
-
.xpath(node, expression, namespaces = {}) ⇒ Object
Methods inherited from Base
actual_native, create_cdata, create_comment, create_declaration, create_doctype, create_element, create_entity_reference, create_namespace, create_processing_instruction, create_text, patch_node, prepare_for_new_document, preprocess_entities, restore_entities, sax_supported?
Methods included from XmlUtils
#encode_entities, #normalize_xml_value, #validate_comment_content, #validate_declaration_encoding, #validate_declaration_standalone, #validate_declaration_version, #validate_element_name, #validate_entity_reference_name, #validate_pi_target, #validate_prefix, #validate_uri
Class Method Details
.add_child(element, child) ⇒ Object
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
|
# File 'lib/moxml/adapter/rexml.rb', line 295
def add_child(element, child)
if element.is_a?(::REXML::Document) && child.is_a?(::REXML::XMLDecl)
attachments.set(element, :xml_declaration, child)
end
case child
when String
element.add_text(child)
append_child_sequence(element, :native)
when ::Moxml::Adapter::CustomizedRexml::EntityReference
refs = attachments.get(element, :entity_refs) || []
refs << child
attachments.set(element, :entity_refs, refs)
append_child_sequence(element, :eref)
else
element.add(child)
append_child_sequence(element, :native)
end
end
|
.add_next_sibling(node, sibling) ⇒ Object
335
336
337
338
|
# File 'lib/moxml/adapter/rexml.rb', line 335
def add_next_sibling(node, sibling)
parent = node.parent
parent.insert_after(node, sibling)
end
|
.add_previous_sibling(node, sibling) ⇒ Object
325
326
327
328
329
330
331
332
333
|
# File 'lib/moxml/adapter/rexml.rb', line 325
def add_previous_sibling(node, sibling)
parent = node.parent
parent.insert_before(node, sibling)
end
|
.append_child_sequence(element, type) ⇒ Object
319
320
321
322
323
|
# File 'lib/moxml/adapter/rexml.rb', line 319
def append_child_sequence(element, type)
seq = attachments.get(element, :child_sequence) || []
seq << type
attachments.set(element, :child_sequence, seq)
end
|
.at_xpath(node, expression, namespaces = {}) ⇒ Object
566
567
568
569
|
# File 'lib/moxml/adapter/rexml.rb', line 566
def at_xpath(node, expression, namespaces = {})
results = xpath(node, expression, namespaces)
results.first
end
|
.attribute_element(attribute) ⇒ Object
261
262
263
|
# File 'lib/moxml/adapter/rexml.rb', line 261
def attribute_element(attribute)
attribute.element
end
|
.attributes(element) ⇒ Object
253
254
255
256
257
258
259
|
# File 'lib/moxml/adapter/rexml.rb', line 253
def attributes(element)
return [] unless element.is_a?(::REXML::Element)
element.attributes.values
.reject { |attr| attr.prefix.to_s.start_with?("xmlns") }
end
|
.cdata_content(node) ⇒ Object
389
390
391
|
# File 'lib/moxml/adapter/rexml.rb', line 389
def cdata_content(node)
node.value
end
|
.children(node) ⇒ Object
183
184
185
186
187
188
189
190
191
192
193
194
195
|
# File 'lib/moxml/adapter/rexml.rb', line 183
def children(node)
return [] unless node.is_a?(::REXML::Parent)
result = node.children.dup
entity_refs = attachments.get(node, :entity_refs)
result.concat(entity_refs) if entity_refs
result.uniq(&:object_id)
end
|
381
382
383
|
# File 'lib/moxml/adapter/rexml.rb', line 381
def (node)
node.string
end
|
.create_document(_native_doc = nil) ⇒ Object
83
84
85
|
# File 'lib/moxml/adapter/rexml.rb', line 83
def create_document(_native_doc = nil)
::REXML::Document.new
end
|
.create_native_cdata(content, _owner_doc = nil) ⇒ Object
103
104
105
|
# File 'lib/moxml/adapter/rexml.rb', line 103
def create_native_cdata(content, _owner_doc = nil)
::REXML::CData.new(content.to_s)
end
|
107
108
109
|
# File 'lib/moxml/adapter/rexml.rb', line 107
def (content, _owner_doc = nil)
::REXML::Comment.new(content.to_s)
end
|
.create_native_declaration(version, encoding, standalone) ⇒ Object
116
117
118
|
# File 'lib/moxml/adapter/rexml.rb', line 116
def create_native_declaration(version, encoding, standalone)
::REXML::XMLDecl.new(version, encoding&.downcase, standalone)
end
|
.create_native_doctype(name, external_id, system_id) ⇒ Object
120
121
122
123
124
125
126
127
128
129
130
131
132
|
# File 'lib/moxml/adapter/rexml.rb', line 120
def create_native_doctype(name, external_id, system_id)
return nil unless name
parts = [name]
if external_id
parts.push("PUBLIC", %("#{external_id}"))
parts << %("#{system_id}") if system_id
elsif system_id
parts.push("SYSTEM", %("#{system_id}"))
end
::REXML::DocType.new(parts.join(" "))
end
|
.create_native_element(name, _owner_doc = nil) ⇒ Object
87
88
89
|
# File 'lib/moxml/adapter/rexml.rb', line 87
def create_native_element(name, _owner_doc = nil)
::REXML::Element.new(name.to_s)
end
|
.create_native_entity_reference(name) ⇒ Object
.create_native_namespace(element, prefix, uri) ⇒ Object
add a namespace definition, keep the element name unchanged
462
463
464
465
|
# File 'lib/moxml/adapter/rexml.rb', line 462
def create_native_namespace(element, prefix, uri)
element.add_namespace(prefix.to_s, uri)
::REXML::Attribute.new(prefix.to_s, uri, element)
end
|
.create_native_processing_instruction(target, content) ⇒ Object
111
112
113
114
|
# File 'lib/moxml/adapter/rexml.rb', line 111
def create_native_processing_instruction(target, content)
::REXML::Instruction.new(target.to_s.dup, content.to_s.dup)
end
|
.create_native_text(content, _owner_doc = nil) ⇒ Object
91
92
93
|
# File 'lib/moxml/adapter/rexml.rb', line 91
def create_native_text(content, _owner_doc = nil)
::REXML::Text.new(content.to_s, true, nil)
end
|
.declaration_attribute(node, name) ⇒ Object
359
360
361
362
363
364
365
366
367
368
|
# File 'lib/moxml/adapter/rexml.rb', line 359
def declaration_attribute(node, name)
case name
when "version"
node.version
when "encoding"
node.encoding
when "standalone"
node.standalone
end
end
|
.doctype_external_id(native) ⇒ Object
523
524
525
|
# File 'lib/moxml/adapter/rexml.rb', line 523
def doctype_external_id(native)
native.public
end
|
.doctype_name(native) ⇒ Object
519
520
521
|
# File 'lib/moxml/adapter/rexml.rb', line 519
def doctype_name(native)
native.name
end
|
.doctype_system_id(native) ⇒ Object
527
528
529
|
# File 'lib/moxml/adapter/rexml.rb', line 527
def doctype_system_id(native)
native.system
end
|
.document(node) ⇒ Object
245
246
247
|
# File 'lib/moxml/adapter/rexml.rb', line 245
def document(node)
node.document
end
|
.duplicate_node(node) ⇒ Object
175
176
177
178
179
180
181
|
# File 'lib/moxml/adapter/rexml.rb', line 175
def duplicate_node(node)
if node.respond_to?(:deep_clone)
node.deep_clone
else
Marshal.load(Marshal.dump(node))
end
end
|
.entity_reference_name(node) ⇒ Object
49
50
51
52
53
54
55
56
57
58
|
# File 'lib/moxml/adapter/rexml.rb', line 49
def (xml)
return "UTF-8" unless xml.start_with?("<?xml")
decl_end = xml.index("?>")
return "UTF-8" unless decl_end
decl = xml[0...decl_end]
match = decl.match(/encoding\s*=\s*["']([^"']+)["']/i)
match ? match[1] : "UTF-8"
end
|
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
|
# File 'lib/moxml/adapter/rexml.rb', line 421
def (element)
return "" unless element
text = ""
element.children.each do |child|
case child
when ::REXML::Text
text += child.value
when ::REXML::Element
child_text = (child)
text += child_text
end
end
text
end
|
.get_attribute(element, name) ⇒ Object
283
284
285
|
# File 'lib/moxml/adapter/rexml.rb', line 283
def get_attribute(element, name)
element.attributes.get_attribute(name)
end
|
.get_attribute_value(element, name) ⇒ Object
287
288
289
|
# File 'lib/moxml/adapter/rexml.rb', line 287
def get_attribute_value(element, name)
element.attributes[name]
end
|
.has_declaration?(native_doc, wrapper) ⇒ Boolean
620
621
622
623
624
625
626
627
628
629
630
631
632
633
|
# File 'lib/moxml/adapter/rexml.rb', line 620
def has_declaration?(native_doc, wrapper)
xml_decl = attachments.get(native_doc, :xml_declaration)
if xml_decl.nil?
if attachments.key?(native_doc, :xml_declaration)
false
else
wrapper.has_xml_declaration
end
else
true
end
end
|
.in_scope_namespaces(element) ⇒ Object
508
509
510
511
512
513
514
515
516
|
# File 'lib/moxml/adapter/rexml.rb', line 508
def in_scope_namespaces(element)
namespaces = {}
element.namespaces.each do |prefix, uri|
key = prefix.to_s.empty? ? "xmlns" : prefix.to_s
ns = ::REXML::Attribute.new(key, uri, element)
namespaces[prefix] = ns
end
namespaces.values
end
|
.inner_text(node) ⇒ Object
441
442
443
444
445
446
447
|
# File 'lib/moxml/adapter/rexml.rb', line 441
def inner_text(node)
text_children = node.children
.grep(::REXML::Text)
.uniq(&:object_id)
text_children.map(&:value).join
end
|
.namespace(node) ⇒ Object
487
488
489
490
491
492
493
494
|
# File 'lib/moxml/adapter/rexml.rb', line 487
def namespace(node)
prefix = node.prefix
uri = node.namespace(prefix)
return if prefix.to_s.empty? && uri.to_s.empty?
owner = node.is_a?(::REXML::Attribute) ? node.element : node
::REXML::Attribute.new(prefix, uri, owner)
end
|
.namespace_definitions(node) ⇒ Object
496
497
498
499
500
501
502
503
504
505
506
|
# File 'lib/moxml/adapter/rexml.rb', line 496
def namespace_definitions(node)
return [] unless node.is_a?(::REXML::Element)
result = []
node.attributes.each_attribute do |attr|
next unless attr.prefix == "xmlns" || (attr.name == "xmlns" && attr.prefix.to_s.empty?)
result << attr
end
result
end
|
.namespace_prefix(node) ⇒ Object
479
480
481
|
# File 'lib/moxml/adapter/rexml.rb', line 479
def namespace_prefix(node)
node.name unless node.name == "xmlns"
end
|
.namespace_uri(node) ⇒ Object
483
484
485
|
# File 'lib/moxml/adapter/rexml.rb', line 483
def namespace_uri(node)
node.value
end
|
.next_sibling(node) ⇒ Object
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
|
# File 'lib/moxml/adapter/rexml.rb', line 201
def next_sibling(node)
current = node.next_sibling
seen = {}
while current
if current.is_a?(::REXML::Text) && current.to_s.strip.empty?
current = current.next_sibling
next
end
if seen[current.object_id]
current = current.next_sibling
next
end
seen[current.object_id] = true
break
end
current
end
|
.node_name(node) ⇒ Object
164
165
166
167
168
169
170
171
172
173
|
# File 'lib/moxml/adapter/rexml.rb', line 164
def node_name(node)
case node
when ::REXML::Element, ::REXML::DocType
node.name
when ::REXML::XMLDecl
"xml"
when ::REXML::Instruction
node.target
end
end
|
.node_type(node) ⇒ Object
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
|
# File 'lib/moxml/adapter/rexml.rb', line 138
def node_type(node)
case node
when ::REXML::Document then :document
when ::REXML::Element then :element
when ::REXML::CData then :cdata
when ::REXML::Text then :text
when ::REXML::Comment then :comment
when ::REXML::Attribute then :attribute when ::REXML::Namespace then :namespace when ::REXML::Instruction then :processing_instruction
when ::REXML::DocType then :doctype
when ::REXML::XMLDecl then :declaration
when ::Moxml::Adapter::CustomizedRexml::EntityReference then :entity_reference
else :unknown
end
end
|
.parent(node) ⇒ Object
197
198
199
|
# File 'lib/moxml/adapter/rexml.rb', line 197
def parent(node)
node.parent
end
|
.parse(xml, options = {}, _context = nil) ⇒ Object
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
|
# File 'lib/moxml/adapter/rexml.rb', line 19
def parse(xml, options = {}, _context = nil)
xml = "" if xml.nil?
processed_xml = if xml.frozen?
xml.dup.force_encoding("UTF-8").encode("UTF-8")
else
xml.force_encoding("UTF-8").encode("UTF-8")
end
processed_xml = preprocess_entities(processed_xml)
native_doc = begin
::REXML::Document.new(processed_xml)
rescue ::REXML::ParseException => e
if options[:strict]
raise Moxml::ParseError.new(
e.message,
line: e.line,
source: xml.is_a?(String) ? xml[0..100] : nil,
)
end
create_document
end
ctx = _context || Context.new(:rexml)
DocumentBuilder.new(ctx).build(native_doc)
end
|
.prepare_xpath_namespaces(node) ⇒ Object
not used at the moment but may be useful when the xpath is upgraded to work with namespaces
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
|
# File 'lib/moxml/adapter/rexml.rb', line 533
def prepare_xpath_namespaces(node)
ns = {}
all_ns = namespace_definitions(node)
all_ns.each do |prefix, uri|
if prefix.to_s.empty?
ns["xmlns"] = uri
else
ns[prefix] = uri
end
end
ns
end
|
.previous_sibling(node) ⇒ Object
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
|
# File 'lib/moxml/adapter/rexml.rb', line 223
def previous_sibling(node)
current = node.previous_sibling
seen = {}
while current
if current.is_a?(::REXML::Text) && current.to_s.strip.empty?
current = current.previous_sibling
next
end
if seen[current.object_id]
current = current.previous_sibling
next
end
seen[current.object_id] = true
break
end
current
end
|
.processing_instruction_content(node) ⇒ Object
401
402
403
|
# File 'lib/moxml/adapter/rexml.rb', line 401
def processing_instruction_content(node)
node.content
end
|
.processing_instruction_target(node) ⇒ Object
397
398
399
|
# File 'lib/moxml/adapter/rexml.rb', line 397
def processing_instruction_target(node)
node.target
end
|
.remove(node) ⇒ Object
340
341
342
343
344
345
346
347
348
|
# File 'lib/moxml/adapter/rexml.rb', line 340
def remove(node)
if node.is_a?(::REXML::XMLDecl) && node.parent.is_a?(::REXML::Document)
attachments.set(node.parent, :xml_declaration, nil)
end
node.remove
end
|
.remove_attribute(element, name) ⇒ Object
291
292
293
|
# File 'lib/moxml/adapter/rexml.rb', line 291
def remove_attribute(element, name)
element.delete_attribute(name.to_s)
end
|
.replace(node, new_node) ⇒ Object
350
351
352
|
# File 'lib/moxml/adapter/rexml.rb', line 350
def replace(node, new_node)
node.replace_with(new_node)
end
|
.replace_children(element, children) ⇒ Object
354
355
356
357
|
# File 'lib/moxml/adapter/rexml.rb', line 354
def replace_children(element, children)
element.children.each(&:remove)
children.each { |child| element.add(child) }
end
|
.root(document) ⇒ Object
249
250
251
|
# File 'lib/moxml/adapter/rexml.rb', line 249
def root(document)
document.root
end
|
.sax_parse(xml, handler) ⇒ void
This method returns an undefined value.
SAX parsing implementation for REXML
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
|
# File 'lib/moxml/adapter/rexml.rb', line 65
def sax_parse(xml, handler)
require "rexml/parsers/sax2parser"
require "rexml/source"
require "stringio"
bridge = REXMLSAX2Bridge.new(handler)
xml_string = xml.is_a?(IO) || xml.is_a?(StringIO) ? xml.read : xml.to_s
source = ::REXML::IOSource.new(StringIO.new(xml_string))
parser = ::REXML::Parsers::SAX2Parser.new(source)
parser.listen(bridge)
parser.parse
rescue ::REXML::ParseException => e
error = Moxml::ParseError.new(e.message, line: e.line)
handler.on_error(error)
end
|
.serialize(node, options = {}) ⇒ Object
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
|
# File 'lib/moxml/adapter/rexml.rb', line 571
def serialize(node, options = {})
output = StringIO.new("") if RUBY_ENGINE == "opal"
output ||= +""
if node.is_a?(::REXML::Document)
should_include_decl = if options.key?(:no_declaration)
!options[:no_declaration]
else
!node.xml_decl.nil?
end
if should_include_decl && node.xml_decl
decl = node.xml_decl
decl.encoding = options[:encoding] if options[:encoding]
output << "<?xml"
output << %( version="#{decl.version}") if decl.version
output << %( encoding="#{decl.encoding}") if decl.encoding
output << %( standalone="#{decl.standalone}") if decl.standalone
output << "?>"
end
node.doctype&.write(output)
node.children.each do |child|
next unless [::REXML::Instruction, ::REXML::CData,
::REXML::Comment, ::REXML::Text].include?(child.class)
write_with_formatter(child, output, options[:indent] || 2)
end
if node.root
write_with_formatter(node.root, output,
options[:indent] || 2)
end
else
write_with_formatter(node, output, options[:indent] || 2)
end
result = output.is_a?(StringIO) ? output.string : output
result.strip
end
|
.set_attribute(element, name, value) ⇒ Object
265
266
267
268
|
# File 'lib/moxml/adapter/rexml.rb', line 265
def set_attribute(element, name, value)
element.attributes[name&.to_s] = value&.to_s
element.attributes.get_attribute(name&.to_s)
end
|
.set_attribute_name(attribute, name) ⇒ Object
270
271
272
273
274
275
276
277
|
# File 'lib/moxml/adapter/rexml.rb', line 270
def set_attribute_name(attribute, name)
old_name = attribute.expanded_name
attribute.name = name
element = attribute.element
element.attributes.delete(old_name)
element.attributes << attribute
end
|
.set_attribute_value(attribute, value) ⇒ Object
279
280
281
|
# File 'lib/moxml/adapter/rexml.rb', line 279
def set_attribute_value(attribute, value)
attribute.normalized = value
end
|
.set_cdata_content(node, content) ⇒ Object
393
394
395
|
# File 'lib/moxml/adapter/rexml.rb', line 393
def set_cdata_content(node, content)
node.value = content.to_s
end
|
385
386
387
|
# File 'lib/moxml/adapter/rexml.rb', line 385
def (node, content)
node.string = content.to_s
end
|
.set_declaration_attribute(node, name, value) ⇒ Object
370
371
372
373
374
375
376
377
378
379
|
# File 'lib/moxml/adapter/rexml.rb', line 370
def set_declaration_attribute(node, name, value)
case name
when "version"
node.version = value
when "encoding"
node.encoding = value
when "standalone"
node.standalone = value
end
end
|
.set_namespace(element, ns) ⇒ Object
add a namespace prefix to the element name AND a namespace definition
468
469
470
471
472
473
474
475
476
477
|
# File 'lib/moxml/adapter/rexml.rb', line 468
def set_namespace(element, ns)
prefix = ns.name.to_s.empty? ? "xmlns" : ns.name.to_s
if element.is_a?(::REXML::Element)
element.add_namespace(prefix,
ns.value)
end
element.name = "#{prefix}:#{element.name}"
owner = element.is_a?(::REXML::Attribute) ? element.element : element
::REXML::Attribute.new(prefix, ns.value, owner)
end
|
.set_node_name(node, name) ⇒ Object
155
156
157
158
159
160
161
162
|
# File 'lib/moxml/adapter/rexml.rb', line 155
def set_node_name(node, name)
case node
when ::REXML::Element
node.name = name.to_s
when ::REXML::Instruction
node.target = name.to_s
end
end
|
.set_processing_instruction_content(node, content) ⇒ Object
405
406
407
|
# File 'lib/moxml/adapter/rexml.rb', line 405
def set_processing_instruction_content(node, content)
node.content = content.to_s
end
|
.set_root(doc, element) ⇒ Object
134
135
136
|
# File 'lib/moxml/adapter/rexml.rb', line 134
def set_root(doc, element)
doc.add_element(element)
end
|
.set_text_content(node, content) ⇒ Object
449
450
451
452
453
454
455
456
457
458
459
|
# File 'lib/moxml/adapter/rexml.rb', line 449
def set_text_content(node, content)
case node
when ::REXML::Text, ::REXML::CData
node.value = content.to_s
when ::REXML::Element
node.texts.each(&:remove)
node.add_text(content.to_s)
end
end
|
.text_content(node) ⇒ Object
.xpath(node, expression, namespaces = {}) ⇒ Object
551
552
553
554
555
556
557
558
559
560
561
562
563
564
|
# File 'lib/moxml/adapter/rexml.rb', line 551
def xpath(node, expression, namespaces = {})
if namespaces && !namespaces.empty?
::REXML::XPath.match(node, expression, namespaces)
else
node.get_elements(expression).to_a
end
rescue ::REXML::ParseException => e
raise Moxml::XPathError.new(
e.message,
expression: expression,
adapter: "REXML",
node: node,
)
end
|