Class: Moxml::Adapter::Rexml

Inherits:
Base
  • Object
show all
Defined in:
lib/moxml/adapter/rexml.rb

Class Method Summary collapse

Methods inherited from Base

create_cdata, create_comment, create_declaration, create_doctype, create_element, create_entity_reference, create_namespace, create_processing_instruction, create_text, entity_reference_name, patch_node, prepare_for_new_document, sax_supported?

Methods included from XmlUtils

#encode_entities, #normalize_xml_value, #validate_comment_content, #validate_declaration_encoding, #validate_declaration_standalone, #validate_declaration_version, #validate_element_name, #validate_entity_reference_name, #validate_pi_target, #validate_prefix, #validate_uri

Class Method Details

.add_child(element, child) ⇒ Object



273
274
275
276
277
278
279
280
281
282
283
284
285
286
# File 'lib/moxml/adapter/rexml.rb', line 273

def add_child(element, child)
  # Special handling for declarations on REXML documents
  if element.is_a?(::REXML::Document) && child.is_a?(::REXML::XMLDecl)
    # Set document's xml_decl directly
    element.instance_variable_set(:@xml_declaration, child)
  end

  case child
  when String
    element.add_text(child)
  else
    element.add(child)
  end
end

.add_next_sibling(node, sibling) ⇒ Object



298
299
300
301
# File 'lib/moxml/adapter/rexml.rb', line 298

def add_next_sibling(node, sibling)
  parent = node.parent
  parent.insert_after(node, sibling)
end

.add_previous_sibling(node, sibling) ⇒ Object



288
289
290
291
292
293
294
295
296
# File 'lib/moxml/adapter/rexml.rb', line 288

def add_previous_sibling(node, sibling)
  parent = node.parent
  # caveat: Rexml fails if children belong to the same parent and are already in a correct order
  # example: "<root><a/><b/></root>"
  # add_previous_sibling(node_b, node_a)
  # result: "<root><b/><a/></root>"
  # expected result: "<root><a/><b/></root>"
  parent.insert_before(node, sibling)
end

.at_xpath(node, expression, namespaces = {}) ⇒ Object



507
508
509
510
# File 'lib/moxml/adapter/rexml.rb', line 507

def at_xpath(node, expression, namespaces = {})
  results = xpath(node, expression, namespaces)
  results.first
end

.attribute_element(attribute) ⇒ Object



239
240
241
# File 'lib/moxml/adapter/rexml.rb', line 239

def attribute_element(attribute)
  attribute.element
end

.attributes(element) ⇒ Object



231
232
233
234
235
236
237
# File 'lib/moxml/adapter/rexml.rb', line 231

def attributes(element)
  return [] unless element.respond_to?(:attributes)

  # Only return non-namespace attributes
  element.attributes.values
    .reject { |attr| attr.prefix.to_s.start_with?("xmlns") }
end

.cdata_content(node) ⇒ Object



352
353
354
# File 'lib/moxml/adapter/rexml.rb', line 352

def cdata_content(node)
  node.value
end

.children(node) ⇒ Object



157
158
159
160
161
162
163
164
165
166
167
168
169
# File 'lib/moxml/adapter/rexml.rb', line 157

def children(node)
  return [] unless node.respond_to?(:children)

  # Get all children and filter out empty text nodes between elements
  result = node.children.reject do |child|
    child.is_a?(::REXML::Text) &&
      child.to_s.strip.empty? &&
      !(child.next_sibling.nil? && child.previous_sibling.nil?)
  end

  # Ensure uniqueness by object_id to prevent duplicates
  result.uniq(&:object_id)
end

.comment_content(node) ⇒ Object



344
345
346
# File 'lib/moxml/adapter/rexml.rb', line 344

def comment_content(node)
  node.string
end

.create_document(_native_doc = nil) ⇒ Object



68
69
70
# File 'lib/moxml/adapter/rexml.rb', line 68

def create_document(_native_doc = nil)
  ::REXML::Document.new
end

.create_native_cdata(content) ⇒ Object



80
81
82
# File 'lib/moxml/adapter/rexml.rb', line 80

def create_native_cdata(content)
  ::REXML::CData.new(content.to_s)
end

.create_native_comment(content) ⇒ Object



84
85
86
# File 'lib/moxml/adapter/rexml.rb', line 84

def create_native_comment(content)
  ::REXML::Comment.new(content.to_s)
end

.create_native_declaration(version, encoding, standalone) ⇒ Object



93
94
95
# File 'lib/moxml/adapter/rexml.rb', line 93

def create_native_declaration(version, encoding, standalone)
  ::REXML::XMLDecl.new(version, encoding&.downcase, standalone)
end

.create_native_doctype(name, external_id, system_id) ⇒ Object



97
98
99
100
101
102
103
104
105
106
107
108
109
# File 'lib/moxml/adapter/rexml.rb', line 97

def create_native_doctype(name, external_id, system_id)
  return nil unless name

  parts = [name]
  if external_id
    parts.push("PUBLIC", %("#{external_id}"))
    parts << %("#{system_id}") if system_id
  elsif system_id
    parts.push("SYSTEM", %("#{system_id}"))
  end

  ::REXML::DocType.new(parts.join(" "))
end

.create_native_element(name) ⇒ Object



72
73
74
# File 'lib/moxml/adapter/rexml.rb', line 72

def create_native_element(name)
  ::REXML::Element.new(name.to_s)
end

.create_native_namespace(element, prefix, uri) ⇒ Object

add a namespace definition, keep the element name unchanged



423
424
425
426
# File 'lib/moxml/adapter/rexml.rb', line 423

def create_native_namespace(element, prefix, uri)
  element.add_namespace(prefix.to_s, uri)
  ::REXML::Attribute.new(prefix.to_s, uri, element)
end

.create_native_processing_instruction(target, content) ⇒ Object



88
89
90
91
# File 'lib/moxml/adapter/rexml.rb', line 88

def create_native_processing_instruction(target, content)
  # Clone strings to avoid frozen string errors
  ::REXML::Instruction.new(target.to_s.dup, content.to_s.dup)
end

.create_native_text(content) ⇒ Object



76
77
78
# File 'lib/moxml/adapter/rexml.rb', line 76

def create_native_text(content)
  ::REXML::Text.new(content.to_s, true, nil)
end

.declaration_attribute(node, name) ⇒ Object



322
323
324
325
326
327
328
329
330
331
# File 'lib/moxml/adapter/rexml.rb', line 322

def declaration_attribute(node, name)
  case name
  when "version"
    node.version
  when "encoding"
    node.encoding
  when "standalone"
    node.standalone
  end
end

.doctype_external_id(native) ⇒ Object



468
469
470
# File 'lib/moxml/adapter/rexml.rb', line 468

def doctype_external_id(native)
  native.public
end

.doctype_name(native) ⇒ Object

Doctype accessor methods



464
465
466
# File 'lib/moxml/adapter/rexml.rb', line 464

def doctype_name(native)
  native.name
end

.doctype_system_id(native) ⇒ Object



472
473
474
# File 'lib/moxml/adapter/rexml.rb', line 472

def doctype_system_id(native)
  native.system
end

.document(node) ⇒ Object



223
224
225
# File 'lib/moxml/adapter/rexml.rb', line 223

def document(node)
  node.document
end

.duplicate_node(node) ⇒ Object



151
152
153
154
155
# File 'lib/moxml/adapter/rexml.rb', line 151

def duplicate_node(node)
  # Make a complete duplicate of the node
  # https://stackoverflow.com/questions/23878384/why-the-original-element-got-changed-when-i-modify-the-copy-created-by-dup-meth
  Marshal.load(Marshal.dump(node))
end

.extract_encoding_from_xml(xml) ⇒ Object



38
39
40
41
42
43
# File 'lib/moxml/adapter/rexml.rb', line 38

def extract_encoding_from_xml(xml)
  # Match XML declaration pattern: <?xml version="..." encoding="..."?>
  # Use atomic group (?>) to prevent polynomial backtracking ReDoS
  match = xml.match(/<\?xml(?>[^>]*)\bencoding\s*=\s*["']([^"']+)["']/i)
  match ? match[1] : "UTF-8"
end

.extract_text_recursively(element) ⇒ Object



382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
# File 'lib/moxml/adapter/rexml.rb', line 382

def extract_text_recursively(element)
  return "" unless element

  text = ""
  element.children.each do |child|
    case child
    when ::REXML::Text
      # Preserve original spacing from text nodes exactly including newlines and all whitespace
      text += child.value
    when ::REXML::Element
      # Extract text recursively from child element
      child_text = extract_text_recursively(child)
      # Concatenate directly like other adapters - NO SPACE INSERTION
      text += child_text
    end
  end
  # Don't strip - preserve original spacing including newlines
  text
end

.get_attribute(element, name) ⇒ Object



261
262
263
# File 'lib/moxml/adapter/rexml.rb', line 261

def get_attribute(element, name)
  element.attributes.get_attribute(name)
end

.get_attribute_value(element, name) ⇒ Object



265
266
267
# File 'lib/moxml/adapter/rexml.rb', line 265

def get_attribute_value(element, name)
  element.attributes[name]
end

.inner_text(node) ⇒ Object



402
403
404
405
406
407
408
# File 'lib/moxml/adapter/rexml.rb', line 402

def inner_text(node)
  # Get direct text children only, filter duplicates
  text_children = node.children
    .grep(::REXML::Text)
    .uniq(&:object_id)
  text_children.map(&:value).join
end

.namespace(node) ⇒ Object



448
449
450
451
452
453
454
455
# File 'lib/moxml/adapter/rexml.rb', line 448

def namespace(node)
  prefix = node.prefix
  uri = node.namespace(prefix)
  return if prefix.to_s.empty? && uri.to_s.empty?

  owner = node.is_a?(::REXML::Attribute) ? node.element : node
  ::REXML::Attribute.new(prefix, uri, owner)
end

.namespace_definitions(node) ⇒ Object



457
458
459
460
461
# File 'lib/moxml/adapter/rexml.rb', line 457

def namespace_definitions(node)
  node.namespaces.map do |prefix, uri|
    ::REXML::Attribute.new(prefix.to_s, uri, node)
  end
end

.namespace_prefix(node) ⇒ Object



440
441
442
# File 'lib/moxml/adapter/rexml.rb', line 440

def namespace_prefix(node)
  node.name unless node.name == "xmlns"
end

.namespace_uri(node) ⇒ Object



444
445
446
# File 'lib/moxml/adapter/rexml.rb', line 444

def namespace_uri(node)
  node.value
end

.next_sibling(node) ⇒ Object



175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
# File 'lib/moxml/adapter/rexml.rb', line 175

def next_sibling(node)
  current = node.next_sibling

  # Skip empty text nodes and duplicates
  seen = Set.new
  while current
    if current.is_a?(::REXML::Text) && current.to_s.strip.empty?
      current = current.next_sibling
      next
    end

    # Check for duplicates
    if seen.include?(current.object_id)
      current = current.next_sibling
      next
    end

    seen.add(current.object_id)
    break
  end

  current
end

.node_name(node) ⇒ Object



140
141
142
143
144
145
146
147
148
149
# File 'lib/moxml/adapter/rexml.rb', line 140

def node_name(node)
  case node
  when ::REXML::Element, ::REXML::DocType
    node.name
  when ::REXML::XMLDecl
    "xml"
  when ::REXML::Instruction
    node.target
  end
end

.node_type(node) ⇒ Object



115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
# File 'lib/moxml/adapter/rexml.rb', line 115

def node_type(node)
  case node
  when ::REXML::Document then :document
  when ::REXML::Element then :element
  when ::REXML::CData then :cdata
  when ::REXML::Text then :text
  when ::REXML::Comment then :comment
  when ::REXML::Attribute then :attribute # but in fact it may be a namespace as well
  when ::REXML::Namespace then :namespace # we don't use this one
  when ::REXML::Instruction then :processing_instruction
  when ::REXML::DocType then :doctype
  when ::REXML::XMLDecl then :declaration
  else :unknown
  end
end

.parent(node) ⇒ Object



171
172
173
# File 'lib/moxml/adapter/rexml.rb', line 171

def parent(node)
  node.parent
end

.parse(xml, options = {}, _context = nil) ⇒ Object



13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
# File 'lib/moxml/adapter/rexml.rb', line 13

def parse(xml, options = {}, _context = nil)
  # Handle frozen strings by creating a mutable copy
  processed_xml = if xml.frozen?
                    xml.dup.force_encoding("UTF-8").encode("UTF-8")
                  else
                    xml.force_encoding("UTF-8").encode("UTF-8")
                  end

  native_doc = begin
    ::REXML::Document.new(processed_xml)
  rescue ::REXML::ParseException => e
    if options[:strict]
      raise Moxml::ParseError.new(
        e.message,
        line: e.line,
        source: xml.is_a?(String) ? xml[0..100] : nil,
      )
    end
    create_document
  end

  ctx = _context || Context.new(:rexml)
  DocumentBuilder.new(ctx).build(native_doc)
end

.prepare_xpath_namespaces(node) ⇒ Object

not used at the moment but may be useful when the xpath is upgraded to work with namespaces



478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
# File 'lib/moxml/adapter/rexml.rb', line 478

def prepare_xpath_namespaces(node)
  ns = {}

  # Get all namespace definitions in scope
  all_ns = namespace_definitions(node)

  # Convert to XPath-friendly format
  all_ns.each do |prefix, uri|
    if prefix.to_s.empty?
      ns["xmlns"] = uri
    else
      ns[prefix] = uri
    end
  end

  ns
end

.previous_sibling(node) ⇒ Object



199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
# File 'lib/moxml/adapter/rexml.rb', line 199

def previous_sibling(node)
  current = node.previous_sibling

  # Skip empty text nodes and duplicates
  seen = Set.new
  while current
    if current.is_a?(::REXML::Text) && current.to_s.strip.empty?
      current = current.previous_sibling
      next
    end

    # Check for duplicates
    if seen.include?(current.object_id)
      current = current.previous_sibling
      next
    end

    seen.add(current.object_id)
    break
  end

  current
end

.processing_instruction_content(node) ⇒ Object



364
365
366
# File 'lib/moxml/adapter/rexml.rb', line 364

def processing_instruction_content(node)
  node.content
end

.processing_instruction_target(node) ⇒ Object



360
361
362
# File 'lib/moxml/adapter/rexml.rb', line 360

def processing_instruction_target(node)
  node.target
end

.remove(node) ⇒ Object



303
304
305
306
307
308
309
310
311
# File 'lib/moxml/adapter/rexml.rb', line 303

def remove(node)
  # Special handling for declarations on REXML documents
  if node.is_a?(::REXML::XMLDecl) && node.parent.is_a?(::REXML::Document)
    # Clear document's xml_declaration when removing declaration
    node.parent.instance_variable_set(:@xml_declaration, nil)
  end

  node.remove
end

.remove_attribute(element, name) ⇒ Object



269
270
271
# File 'lib/moxml/adapter/rexml.rb', line 269

def remove_attribute(element, name)
  element.delete_attribute(name.to_s)
end

.replace(node, new_node) ⇒ Object



313
314
315
# File 'lib/moxml/adapter/rexml.rb', line 313

def replace(node, new_node)
  node.replace_with(new_node)
end

.replace_children(element, children) ⇒ Object



317
318
319
320
# File 'lib/moxml/adapter/rexml.rb', line 317

def replace_children(element, children)
  element.children.each(&:remove)
  children.each { |child| element.add(child) }
end

.root(document) ⇒ Object



227
228
229
# File 'lib/moxml/adapter/rexml.rb', line 227

def root(document)
  document.root
end

.sax_parse(xml, handler) ⇒ void

This method returns an undefined value.

SAX parsing implementation for REXML

Parameters:



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
# File 'lib/moxml/adapter/rexml.rb', line 50

def sax_parse(xml, handler)
  require "rexml/parsers/sax2parser"
  require "rexml/source"
  require "stringio"

  bridge = REXMLSAX2Bridge.new(handler)

  xml_string = xml.respond_to?(:read) ? xml.read : xml.to_s
  source = ::REXML::IOSource.new(StringIO.new(xml_string))

  parser = ::REXML::Parsers::SAX2Parser.new(source)
  parser.listen(bridge)
  parser.parse
rescue ::REXML::ParseException => e
  error = Moxml::ParseError.new(e.message, line: e.line)
  handler.on_error(error)
end

.serialize(node, options = {}) ⇒ Object



512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
# File 'lib/moxml/adapter/rexml.rb', line 512

def serialize(node, options = {})
  output = +""

  if node.is_a?(::REXML::Document)
    # Check if we should include declaration
    # Priority: explicit option > check if document has xml_decl
    should_include_decl = if options.key?(:no_declaration)
                            !options[:no_declaration]
                          else
                            # Include declaration only if document has xml_decl
                            !node.xml_decl.nil?
                          end

    # Include XML declaration only if should_include_decl and xml_decl exists
    if should_include_decl && node.xml_decl
      decl = node.xml_decl
      decl.encoding = options[:encoding] if options[:encoding]
      output << "<?xml"
      output << %( version="#{decl.version}") if decl.version
      output << %( encoding="#{decl.encoding}") if decl.encoding
      output << %( standalone="#{decl.standalone}") if decl.standalone
      output << "?>"
    end

    # output << "\n"
    node.doctype&.write(output)

    # Write processing instructions
    node.children.each do |child|
      next unless [::REXML::Instruction, ::REXML::CData,
                   ::REXML::Comment, ::REXML::Text].include?(child.class)

      write_with_formatter(child, output, options[:indent] || 2)
      # output << "\n"
    end

    if node.root
      write_with_formatter(node.root, output,
                           options[:indent] || 2)
    end
  else
    write_with_formatter(node, output, options[:indent] || 2)
  end

  output.strip
end

.set_attribute(element, name, value) ⇒ Object



243
244
245
246
# File 'lib/moxml/adapter/rexml.rb', line 243

def set_attribute(element, name, value)
  element.attributes[name&.to_s] = value&.to_s
  element.attributes.get_attribute(name&.to_s)
end

.set_attribute_name(attribute, name) ⇒ Object



248
249
250
251
252
253
254
255
# File 'lib/moxml/adapter/rexml.rb', line 248

def set_attribute_name(attribute, name)
  old_name = attribute.expanded_name
  attribute.name = name
  # Rexml doesn't change the keys of the attributes hash
  element = attribute.element
  element.attributes.delete(old_name)
  element.attributes << attribute
end

.set_attribute_value(attribute, value) ⇒ Object



257
258
259
# File 'lib/moxml/adapter/rexml.rb', line 257

def set_attribute_value(attribute, value)
  attribute.normalized = value
end

.set_cdata_content(node, content) ⇒ Object



356
357
358
# File 'lib/moxml/adapter/rexml.rb', line 356

def set_cdata_content(node, content)
  node.value = content.to_s
end

.set_comment_content(node, content) ⇒ Object



348
349
350
# File 'lib/moxml/adapter/rexml.rb', line 348

def set_comment_content(node, content)
  node.string = content.to_s
end

.set_declaration_attribute(node, name, value) ⇒ Object



333
334
335
336
337
338
339
340
341
342
# File 'lib/moxml/adapter/rexml.rb', line 333

def set_declaration_attribute(node, name, value)
  case name
  when "version"
    node.version = value
  when "encoding"
    node.encoding = value
  when "standalone"
    node.standalone = value
  end
end

.set_namespace(element, ns) ⇒ Object

add a namespace prefix to the element name AND a namespace definition



429
430
431
432
433
434
435
436
437
438
# File 'lib/moxml/adapter/rexml.rb', line 429

def set_namespace(element, ns)
  prefix = ns.name.to_s.empty? ? "xmlns" : ns.name.to_s
  if element.respond_to?(:add_namespace)
    element.add_namespace(prefix,
                          ns.value)
  end
  element.name = "#{prefix}:#{element.name}"
  owner = element.is_a?(::REXML::Attribute) ? element.element : element
  ::REXML::Attribute.new(prefix, ns.value, owner)
end

.set_node_name(node, name) ⇒ Object



131
132
133
134
135
136
137
138
# File 'lib/moxml/adapter/rexml.rb', line 131

def set_node_name(node, name)
  case node
  when ::REXML::Element
    node.name = name.to_s
  when ::REXML::Instruction
    node.target = name.to_s
  end
end

.set_processing_instruction_content(node, content) ⇒ Object



368
369
370
# File 'lib/moxml/adapter/rexml.rb', line 368

def set_processing_instruction_content(node, content)
  node.content = content.to_s
end

.set_root(doc, element) ⇒ Object



111
112
113
# File 'lib/moxml/adapter/rexml.rb', line 111

def set_root(doc, element)
  doc.add_element(element)
end

.set_text_content(node, content) ⇒ Object



410
411
412
413
414
415
416
417
418
419
420
# File 'lib/moxml/adapter/rexml.rb', line 410

def set_text_content(node, content)
  case node
  when ::REXML::Text, ::REXML::CData
    node.value = content.to_s
  when ::REXML::Element
    # Remove existing text nodes to prevent duplicates
    node.texts.each(&:remove)
    # Add new text content
    node.add_text(content.to_s)
  end
end

.text_content(node) ⇒ Object



372
373
374
375
376
377
378
379
380
# File 'lib/moxml/adapter/rexml.rb', line 372

def text_content(node)
  case node
  when ::REXML::Text, ::REXML::CData
    node.value.to_s
  when ::REXML::Element
    # Extract text recursively from all children to match other adapters
    extract_text_recursively(node)
  end
end

.xpath(node, expression, _namespaces = {}) ⇒ Object



496
497
498
499
500
501
502
503
504
505
# File 'lib/moxml/adapter/rexml.rb', line 496

def xpath(node, expression, _namespaces = {})
  node.get_elements(expression).to_a
rescue ::REXML::ParseException => e
  raise Moxml::XPathError.new(
    e.message,
    expression: expression,
    adapter: "REXML",
    node: node,
  )
end